// ESTView.cpp : implementation of the CESTView class
//
#include "stdafx.h"
#include "BAMBE.h"

#include "ESTView.h"
#include "LogScrollView.h"
#include "CloseWins.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CESTView

IMPLEMENT_DYNCREATE(CESTView, CScrollView)

BEGIN_MESSAGE_MAP(CESTView, CScrollView)
	//{{AFX_MSG_MAP(CESTView)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_COMMAND(ID_RUN_STOP, OnRunStop)
	ON_UPDATE_COMMAND_UI(ID_RUN_STOP, OnUpdateRunStop)
	ON_COMMAND(ID_RUN_PAUSE, OnRunPause)
	ON_UPDATE_COMMAND_UI(ID_RUN_PAUSE, OnUpdateRunPause)
	ON_COMMAND(ID_RUN_RESUME, OnRunResume)
	ON_UPDATE_COMMAND_UI(ID_RUN_RESUME, OnUpdateRunResume)
	ON_COMMAND(ID_RUN_RUN, OnRunRun)
	ON_UPDATE_COMMAND_UI(ID_RUN_RUN, OnUpdateRunRun)
	ON_COMMAND(ID_CLOSEALLWINS, OnCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_CLOSEALLWINS, OnUpdateCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_WM_SIZE()
	ON_COMMAND(ID_FORMAT_FONT, OnFormatFont)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_USERINVALEST, OnUserInvalEST)
	ON_MESSAGE(WM_USERCLOSEEST, OnUserCloseEST)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CESTView construction/destruction

CESTView::CESTView() : m_ViewCharSize(0,0), m_DocSize(500, 500)
{

	m_pFont = NULL;
	m_nLinesPerPrintPage= 0;
	m_nPrintPages		= 0;
}

CESTView::~CESTView()
{
	delete m_pFont;
}

BOOL CESTView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CESTView drawing

void CESTView::OnDraw(CDC* pDC)
{
	int nFirstLn, nLastLn, cont;
  
	cont = ComputeVisibleLines(pDC, nFirstLn, nLastLn);
	if (cont)
	{
		int nYPos = - nFirstLn * GetCharSize().cy;
		int nXPos = 2 * GetCharSize().cx;

		OnDraw(pDC, nFirstLn, nLastLn,nXPos,nYPos);
	}
}

void CESTView::OnUpdate()
{
	ComputeViewMetrics();
  
	SetScrollSizes(MM_LOENGLISH, GetDocSize());
	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CESTView printing

BOOL CESTView::OnPreparePrinting(CPrintInfo* pInfo)
{
	ComputePrintMetrics();

   	pInfo->SetMaxPage(GetPrintPageCount()); 

	//	Disable print to file
//	pInfo->m_pPD->m_pd.Flags |= PD_DISABLEPRINTTOFILE;

	//	display the print dialog
	BOOL bRet = DoPreparePrinting(pInfo);

	//	Set number of pages to preview, this needs to be set
	//	after calling DoPreparePrinting() which sets this
	//	based on ini file settings
	pInfo->m_nNumPreviewPages = 1;  

	return (bRet);
}

void CESTView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
/*	int pageHt = pDC->GetDeviceCaps(VERTRES);
	int logPixelsY = pDC->GetDeviceCaps(LOGPIXELSY);// finds dots per inch
	int numInches = (int)(m_DocSize.cy / 100);
	int numPages = (numInches * logPixelsY) / pageHt + 1;
	pInfo->SetMaxPage(numPages);*/

	ComputePrintMetrics();
}

void CESTView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CESTView diagnostics

#ifdef _DEBUG
void CESTView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CESTView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CGenericDoc* CESTView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGenericDoc)));
	return (CGenericDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CESTView message handlers

CFont* CESTView::GetFont()
{
	if(m_pFont == NULL) {
		m_pFont = new CFont;
		if(m_pFont) {
			//	Default to 10 pt Arial
			m_pFont->CreatePointFont(100, "Courier New");
		}
	}
	return m_pFont;
}

void CESTView::ComputeViewMetrics()
{	
		// get a CDC* for the screen
	CDC* pDC = CDC::FromHandle(::GetDC(NULL));
	int nSaveDC = pDC->SaveDC();
  
		// select mapping mode
	pDC->SetMapMode(MM_LOENGLISH); 
  
		// select the font and get its metrics
	CFont* pPreviousFont = pDC->SelectObject(GetFont());
	TEXTMETRIC tm;
	pDC->GetTextMetrics(&tm);
  
		//	Calculate view character size
	m_ViewCharSize.cy = tm.tmHeight + tm.tmExternalLeading;
	m_ViewCharSize.cx = tm.tmAveCharWidth; 
  
		// convert to device units to minimize round off error
	pDC->LPtoDP(&m_ViewCharSize);
  
		//	Calculate document size
	if (GetLineList()) {
		m_DocSize.cy = m_ViewCharSize.cy * GetLineList()->GetCount();
  
		// loop through the document and find the longest line
	CString	Line;
	CSize size;
	POSITION pos = GetLineList()->GetHeadPosition();
	while(pos != NULL) {
		Line = GetLineList()->GetNext( pos );
		size = pDC->GetTextExtent(Line, Line.GetLength());
		m_DocSize.cx = max(size.cx, m_DocSize.cx);
	}
  
		//	Account for our simple margin
	m_DocSize.cx += 4 * m_ViewCharSize.cx;
	}
  
		// clean up
	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}
	pDC->RestoreDC(nSaveDC);
	::ReleaseDC(NULL,pDC->GetSafeHdc());
}

int CESTView::ComputeVisibleLines(CDC* pDC, int& nFirst, int& nLast)
{
	if (GetLineList() == NULL)
		return 0;

	int nLineCount = GetLineList()->GetCount();
  
		// Get the viewport origin, convert to logical coordinates
	CPoint pt = pDC->GetViewportOrg();
	pDC->DPtoLP(&pt,1);
	  
		// Get the clipping region, in logical coordinates
	CRect rc;   
	pDC->GetClipBox(&rc);
  
		// Get the logical line height
	CSize CharSize = GetCharSize();
	
		// Compute the first visible line
	nFirst = min(abs((rc.top - pt.y)/CharSize.cy), 
				nLineCount-1);
	   
		// compute the last visible line
	nLast = min(abs(rc.Height())/CharSize.cy + nFirst + 1, 
				nLineCount-1); 
	return 1;
}

void CESTView::OnDraw(CDC* pDC, int nFirstLn, int nLastLn, 
						int nXPos /*= 0*/, int nYPos /*= 0*/)
{
		// 	Select specified font
	CFont* pPreviousFont = pDC->SelectObject(GetFont());
  
		//	Needed for height of each line
	CSize CharSize = GetCharSize();
  
		//	Get list of strings from the document
		//	and ESTput them to the display context
	CStringList *pLineList = GetLineList();
  
	CString 	strLine;
	POSITION 	pos;
	while (nFirstLn <= nLastLn) {
		if( ( pos = pLineList->FindIndex( nFirstLn )) != NULL ){
			strLine = pLineList->GetAt(pos); 
			pDC->TabbedTextOut(nXPos, nYPos, strLine, 0, NULL, 0);
			nYPos -= CharSize.cy;
			nFirstLn++;
		}
	}
  
		//	Cleanup and restore original GDI Objects
	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}
}

void CESTView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();

	ComputeViewMetrics();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());

	pBApp->globals.g_ESTHwnd = GetSafeHwnd();
}

LRESULT CESTView::OnUserInvalEST(WPARAM wParam, LPARAM lParam)
{
	OnUpdate();
	return 1;
}

LRESULT CESTView::OnUserCloseEST(WPARAM wParam, LPARAM lParam)
{
	CGenericDoc* pDoc = GetDocument();
	pDoc->OnCloseDocument();
	return 1;
}

void CESTView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CESTView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CESTView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CESTView::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CESTView::OnRunStop() 
{
}

void CESTView::OnUpdateRunStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CESTView::OnRunPause() 
{
}

void CESTView::OnUpdateRunPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CESTView::OnRunResume() 
{
}

void CESTView::OnUpdateRunResume(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CESTView::OnRunRun() 
{
}

void CESTView::OnUpdateRunRun(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

/*
void CESTView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	CScrollView::OnPrepareDC(pDC, pInfo);
	if (pDC->IsPrinting()) {
		int pageHt = pDC->GetDeviceCaps(VERTRES);
		int originY = pageHt * (pInfo->m_nCurPage - 1);
		pDC->SetViewportOrg(0, -originY);
	}
}
*/
void CESTView::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);			
}

void CESTView::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);			
}

void CESTView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	int nFirstLn, nLastLn;
	ComputePrintableLines(pDC, pInfo, nFirstLn, nLastLn);
	int nXPos = 2 * GetCharSize().cx;	// left margin
	int nYPos = - 2 * GetCharSize().cy;	// top margin
	OnDraw(pDC, nFirstLn, nLastLn, nXPos, nYPos);
	
//	CScrollView::OnPrint(pDC, pInfo);
}

CDC* CESTView::CreatePrinterDC()
{
	ASSERT_VALID(this);
	PRINTDLG PrtDlg;
	HDC      hDC;
   
	if (!AfxGetApp()->GetPrinterDeviceDefaults(&PrtDlg))
	{
		hDC = ::CreateDC("display",NULL,NULL,NULL);
	} 
	else
	{
		CPrintDialog dlg(FALSE); 
		dlg.m_pd.hDevMode = PrtDlg.hDevMode;
		dlg.m_pd.hDevNames = PrtDlg.hDevNames;
		hDC = dlg.CreatePrinterDC();
	}
      
	CDC* pDC = CDC::FromHandle(hDC);
	// This is a printer DC, so set m_bPrinting
	// this is necessary so CScrollView::OnPrepareDC won't modify the 
	// ViewportOrg, and cause LPtoDP to return an inappropriate result.
	pDC->m_bPrinting = TRUE;              
	return pDC;        
}

void CESTView::ComputePrintMetrics()
{  
   	// Get a printer CDC*
   	CDC* pDC = CreatePrinterDC();
   
   	// Select mapping mode and font
   	pDC->SetMapMode(MM_LOENGLISH);
   	CFont* pPreviousFont = pDC->SelectObject(GetFont());
   
   	TEXTMETRIC tm;
   	pDC->GetTextMetrics(&tm);
   
    //	Calculate printable character size
   	m_PrintCharSize.cy = tm.tmHeight + tm.tmExternalLeading;
   	m_PrintCharSize.cx = tm.tmAveCharWidth;   
  
	//  convert to device units to minimize round off error
	//	when calculating lines per page
	pDC->LPtoDP(&m_PrintCharSize);                                   

	//	Calculate number of printable pages and line per page
   	//	Get Physical page size in device units   
	int nPhysDevHeight = pDC->GetDeviceCaps(PHYSICALHEIGHT);
	int nPhysOffsetY = pDC->GetDeviceCaps(PHYSICALOFFSETY);
	nPhysDevHeight -= nPhysOffsetY * 2;
   
   	//  Lines per print page  
   	m_nLinesPerPrintPage = abs( nPhysDevHeight/m_PrintCharSize.cy ) - 8;  
  
	// with a bottom margin...
	m_nLinesPerPrintPage -= 3;

   	//  Number of print pages
   	m_nPrintPages = GetLineList()->GetCount() / m_nLinesPerPrintPage; 
   	if (GetLineList()->GetCount() % m_nLinesPerPrintPage) 
   	{
   	    m_nPrintPages++;
   	}

   	// clean up                         
	if(pPreviousFont)
	{
   		pDC->SelectObject(pPreviousFont);
	}
   	::DeleteDC(pDC->GetSafeHdc()); 
} 

void CESTView::ComputePrintableLines( CDC* pDC, CPrintInfo* pInfo,
                                       int& nFirst, int& nLast)
{                                        
	nFirst = GetLinesPerPrintPage() * (pInfo->m_nCurPage-1);
	nLast  = min( nFirst + GetLinesPerPrintPage() - 1, //was 1
                 GetLineList()->GetCount() - 1);
}

void CESTView::OnFormatFont() 
{
	CFont* pFont = GetFont();

	LOGFONT	lf;
	pFont->GetObject(sizeof(LOGFONT), &lf);
	
	CFontDialog	dlg(&lf, CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT);

	if(dlg.DoModal() == IDOK) {
		if(m_pFont) {
			delete m_pFont;
		}

		m_pFont = new CFont;
		if(m_pFont){
			m_pFont->CreateFontIndirect(&lf);
		}

		//	This will cause OnUpdate() to be called ensuring
		//	that our cached metrics and scrolling get updated
		GetDocument()->UpdateAllViews(NULL);
	}
}

void CESTView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);
	
	OnUpdate();	
}

void CESTView::OnCloseAllWins()
{
	CCloseWins printmssg;
	int ans = printmssg.DoModal();

	if(ans == IDOK)
		pBApp->globals.CloseAllWins();
}

void CESTView::OnUpdateCloseAllWins(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}

CStringList* CESTView::GetLineList()
{
	return pBApp->globals.EstList;
}