// KAP2View.cpp : implementation of the CKAP2View class
//

#include "stdafx.h"
#include "BAMBE.h"
#include "Window.h"

#include "KAP2View.h"
#include "LogScrollView.h"
#include "CloseWins.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern Windows win;
extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CKAP2View

IMPLEMENT_DYNCREATE(CKAP2View, CScrollView)

BEGIN_MESSAGE_MAP(CKAP2View, CScrollView)
	//{{AFX_MSG_MAP(CKAP2View)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_WM_HSCROLL()
	ON_COMMAND(ID_RUN_STOP, OnRunStop)
	ON_UPDATE_COMMAND_UI(ID_RUN_STOP, OnUpdateRunStop)
	ON_COMMAND(ID_RUN_PAUSE, OnRunPause)
	ON_UPDATE_COMMAND_UI(ID_RUN_PAUSE, OnUpdateRunPause)
	ON_COMMAND(ID_RUN_RESUME, OnRunResume)
	ON_UPDATE_COMMAND_UI(ID_RUN_RESUME, OnUpdateRunResume)
	ON_COMMAND(ID_RUN_RUN, OnRunRun)
	ON_UPDATE_COMMAND_UI(ID_RUN_RUN, OnUpdateRunRun)
	ON_COMMAND(ID_CLOSEALLWINS, OnCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_CLOSEALLWINS, OnUpdateCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_USERINVALKAP2, OnUserInvalKap2)
	ON_MESSAGE(WM_USERCLOSEKAP2, OnUserCloseKap2)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKAP2View construction/destruction

CKAP2View::CKAP2View() : m_DocSize(100, 180)
{
	kapArCt = curCyCt = 0;
	m_pFont = NULL;
	IsDrawing = FALSE;
}

CKAP2View::~CKAP2View()
{
	delete m_pFont;
}

BOOL CKAP2View::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CKAP2View drawing

void CKAP2View::OnDraw(CDC* pDC)
{
	IsDrawing = TRUE;

	pDC->SetMapMode(MM_LOENGLISH);
	CFont* pPreviousFont = pDC->SelectObject(GetFont());

	kapArCt = win.getKACt();
	curCyCt = win.getCurCCt();

	int header = 0;
	if (pDC->IsPrinting()) {
		PrintHeader(pDC);
		header = -60; 
	}

	if ((win.getminkap() != 0) && (win.getmaxkap() >= win.getminkap())) {
		PrintVertScale(pDC, header);
	}

	int nFirst, nLast, ok;
	if (!pBApp->globals.RunAvail) {
		ok = ComputeVisibleLines(pDC, nFirst, nLast);
		if(!ok) {
			nFirst = 0;
			nLast = 0;
		}
	}
	else {
		nFirst = 0;
		nLast = kapArCt - 1;
	}

	double xscale = 2.5 * 1.33;
	
	PrintHorzScale(pDC, nFirst, nLast, xscale, header);

	Perfmeter(pDC, nFirst, nLast, xscale, header);

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}

	if (!pBApp->globals.RunAvail) {
		HWND hWnd = GetSafeHwnd();
		int max = GetScrollLimit(SB_HORZ);
		::SetScrollPos(hWnd, SB_HORZ, max, TRUE);
	}

	IsDrawing = FALSE;
}

void CKAP2View::PrintHeader(CDC * pDC)
{
	CString tmp;
	CGenericDoc* pDoc = GetDocument();
	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch
	double pi = logPixels / 100;

	double posx, mp, bot;
	bot = (1.0/pi);
	mp = (4.0/3.0)/bot;

	CFont* pPreviousFont = pDC->SelectObject(GetFont());
	pDC->SetMapMode(MM_LOENGLISH);

	for (int i = m_npage; i <= m_nMax; i++) { 
		tmp.Format("%s     -     Page %d ", (LPCSTR) pDoc->GetTitle(), i);
		posx = 20 + ((i-1) * (pageWd/mp) * (4.0/3.0));
 		pDC->TextOut(posx, -20, tmp);
	}

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}

}

void CKAP2View::PrintVertScale(CDC * pDC, int header)
{
	CString tmp;
	double posx = 0, posy = 0,
		Range, unit, mark, step;

	CRect rc;   
	pDC->GetClipBox(&rc);
	posx = rc.left;

	Range = fabs(win.getmaxkap() - win.getminkap()) + .5;
	unit = Range / 100;
	mark = win.getminkap();
	step = unit * 20;

	for (int i = 0; i <= 5; i++) {
		tmp.Format("%.1f", mark);
		posy = -1.33 * (100 - (i*20));
		pDC->TextOut(posx, posy + header, tmp);
		mark += step;
	}
}

void CKAP2View::PrintHorzScale(CDC * pDC, int nFirst, int nLast,
							   double xscale, int header)
{
	CString tmp;
	double posx = 0, posy = header;
	int j, value;

	if(curCyCt) {
		posx = (20 * 1.33) + (nFirst * xscale);
		for (j = nFirst; j <= nLast; j++) {
			value = win.getCurCycle(j);
			if ((j % 8) == 0) {
		 		tmp.Format("|");
				pDC->TextOut(posx, posy - 143, tmp);
				tmp.Format("%d", value);
				if ((j % 16) == 0) 
					pDC->TextOut(posx, posy - 154, tmp);
				else
					pDC->TextOut(posx, posy - 164, tmp);
			}
			posx += xscale;
		} // end for
	}
}

void CKAP2View::Perfmeter(CDC* pDC, int nFirst, int nLast,
						  double xscale, int header)
{
	double posx = 0, posy;
	int count = kapArCt, totalk;
	double Range, diff, unit, value;

	TRACE("Perfmeter - count = %d\n", count);
	if (kapArCt) {
		CPen* pen;
		CPen* oldPen;// = pDC->SelectObject(&pen);

		if (pBApp->globals.single_kappa)
			totalk = 1;
		else totalk = pBApp->globals.num_tags;

	  for (int line = 0; line < totalk; line++) {
		  pen = new CPen;
		switch (line) {
		  case 0: pen->CreatePen(PS_SOLID,1,RGB(160,0,0));	break;		// medium-dark red
		  case 1: pen->CreatePen(PS_SOLID, 1, RGB(0,0,160)); break;		// medium-dark blue
		  case 2: pen->CreatePen(PS_SOLID, 1, RGB(0,160,0)); break;		// medium green
		  case 3: pen->CreatePen(PS_SOLID, 1, RGB(255,255,0)); break;	// yellow
	      case 4: pen->CreatePen(PS_SOLID, 1, RGB(255,0,255)); break;	// magenta (bright)
		  case 5: pen->CreatePen(PS_SOLID, 1, RGB(153,153,153)); break;	// gray
		  case 6: pen->CreatePen(PS_SOLID, 1, RGB(175,181,85)); break;	// light brown
		  case 7: pen->CreatePen(PS_SOLID, 1, RGB(255,0,0)); break;		// bright red
		  case 8: pen->CreatePen(PS_SOLID, 1, RGB(255,146,41)); break;	// light orange
		  case 9: pen->CreatePen(PS_SOLID, 1, RGB(0,0,0)); break;		// black
		}
		oldPen = pDC->SelectObject(pen);

			// Move the cursor to the first point on the graph
		posx = (22 * 1.33) + (nFirst * xscale);

		// first find the basis for caluclations
	    Range = fabs(win.getmaxkap() - win.getminkap()) + .5;
		unit = Range / 100;

		value =(double) win.getKappaArray(line, nFirst) / 1000;
		diff = fabs(value - win.getminkap());

		posy = ((100 - (diff / unit)) * -1.33) - 6;
		pDC->MoveTo(posx, posy + header);

		// Draw the lines on the screen
			for (int ct = nFirst+1; ct <= nLast; ct++){
				value = (double)win.getKappaArray(line, ct) / 1000;
				diff = fabs(value - win.getminkap());
				posy = ((100 - (diff / unit)) * -1.33) - 6;
				posx += xscale;
				pDC->LineTo(posx, posy + header);
			} // for
//		} // else

		pDC->SelectObject(oldPen);
		DeleteObject(pen);
		delete pen;
	  } // for
	} // if
}

void CKAP2View::OnUpdate()
{
	ComputeViewMetrics();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());

	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CKAP2View printing

BOOL CKAP2View::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CKAP2View::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch

	int numInches = (int)(m_DocSize.cx / 100);

	int numPages = (numInches * logPixels) / pageWd + 1;

	pInfo->SetMaxPage(numPages);
}

void CKAP2View::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CKAP2View diagnostics

#ifdef _DEBUG
void CKAP2View::AssertValid() const
{
	CScrollView::AssertValid();
}

void CKAP2View::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CGenericDoc* CKAP2View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGenericDoc)));
	return (CGenericDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CKAP2View message handlers

CFont *	CKAP2View::GetFont()
{
	if(m_pFont == NULL) {
		m_pFont = new CFont;
		if(m_pFont) {
			//	Default to 10 pt Arial //was 100
			m_pFont->CreatePointFont(80, "Courier New");
		}
	}
	return m_pFont;
}

void CKAP2View::ComputeViewMetrics()
{
	if(win.getKACt())
		m_DocSize.cx = (2.5 * 1.33) * win.getKACt() + (22 * 1.33);
}

int CKAP2View::ComputeVisibleLines(CDC* pDC, int& nFirst, int& nLast)
{
	if (!kapArCt)
		return 0;

	int nCount = kapArCt;
    
		// Get the clipping region, in logical coordinates
	CRect rc;   
	pDC->GetClipBox(&rc);
	
		// Compute the first visible line
	if (rc.Width() < 150) {
		if (nCount <= 25)
			nFirst = 0; 
		else 
			nFirst = nCount - 25;
	}
	else nFirst = 0;
		// Compute the last visible line
	nLast =	nCount - 1;
	
	return 1;
}

void CKAP2View::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());

	ComputeViewMetrics();
	pBApp->globals.g_Kap2Hwnd = GetSafeHwnd();
}

LRESULT CKAP2View::OnUserInvalKap2(WPARAM wParam, LPARAM lParam)
{
	if (!IsDrawing)
		OnUpdate();

	return 1;
}

LRESULT CKAP2View::OnUserCloseKap2(WPARAM wParam, LPARAM lParam)
{
	CGenericDoc* pDoc = GetDocument();
	pDoc->OnCloseDocument();	
	return 1;
}
void CKAP2View::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CKAP2View::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CKAP2View::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);		
}

void CKAP2View::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);		
}

void CKAP2View::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
	OnUpdate();
}

void CKAP2View::OnRunStop() 
{
	pBApp->globals.IsStop = TRUE;
	pBApp->globals.RunAvail = TRUE;
	pBApp->globals.IsPause = FALSE;
}

void CKAP2View::OnUpdateRunStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.RunAvail);			
}

void CKAP2View::OnRunPause() 
{
	pBApp->globals.IsPause = TRUE;	
}

void CKAP2View::OnUpdateRunPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.IsPause && !pBApp->globals.RunAvail);			
}

void CKAP2View::OnRunResume() 
{
	pBApp->globals.IsPause = FALSE;	
}

void CKAP2View::OnUpdateRunResume(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.IsPause && !pBApp->globals.RunAvail);		
}

void CKAP2View::OnRunRun() 
{
	pBApp->globals.g_pView->SendMessage(WM_USERCHILDRUN);		
}

void CKAP2View::OnUpdateRunRun(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail && pBApp->globals.DocIsOpen);			
}

void CKAP2View::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if (pDC->IsPrinting()) {
		int pageWd = pDC->GetDeviceCaps(HORZRES);
		int originX = pageWd * (pInfo->m_nCurPage - 1);
		pDC->SetViewportOrg(-originX, 0);
		CView::OnPrepareDC(pDC, pInfo);
	}
	else {
		CScrollView::OnPrepareDC(pDC, pInfo);
	}
}

void CKAP2View::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);		
}

void CKAP2View::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
		pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);		
	
}

void CKAP2View::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	m_npage = pInfo->m_nCurPage;	
	m_nMax = pInfo->GetMaxPage();
	CScrollView::OnPrint(pDC, pInfo);
}

void CKAP2View::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);
	
	OnUpdate();	
}

void CKAP2View::OnCloseAllWins()
{
	CCloseWins printmssg;
	int ans = printmssg.DoModal();

	if(ans == IDOK)
		pBApp->globals.CloseAllWins();
}

void CKAP2View::OnUpdateCloseAllWins(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}