// LPDView.cpp : implementation of the CLPDView class
//

#include "stdafx.h"
#include "BAMBE.h"
#include "Window.h"

#include "LPDView.h"
#include "LogScrollView.h"
#include "CloseWins.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern Windows win;
extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CLPDView

IMPLEMENT_DYNCREATE(CLPDView, CScrollView)

BEGIN_MESSAGE_MAP(CLPDView, CScrollView)
	//{{AFX_MSG_MAP(CLPDView)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_WM_HSCROLL()
	ON_COMMAND(ID_RUN_STOP, OnRunStop)
	ON_UPDATE_COMMAND_UI(ID_RUN_STOP, OnUpdateRunStop)
	ON_COMMAND(ID_RUN_PAUSE, OnRunPause)
	ON_UPDATE_COMMAND_UI(ID_RUN_PAUSE, OnUpdateRunPause)
	ON_COMMAND(ID_RUN_RESUME, OnRunResume)
	ON_UPDATE_COMMAND_UI(ID_RUN_RESUME, OnUpdateRunResume)
	ON_COMMAND(ID_RUN_RUN, OnRunRun)
	ON_UPDATE_COMMAND_UI(ID_RUN_RUN, OnUpdateRunRun)
	ON_COMMAND(ID_CLOSEALLWINS, OnCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_CLOSEALLWINS, OnUpdateCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_USERINVALLPD, OnUserInvalLpd)
	ON_MESSAGE(WM_USERCLOSELPD, OnUserCloseLpd)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLPDView construction/destruction

CLPDView::CLPDView() : m_DocSize(100, 180)
{
	curCyCt = curCount = 0;
	m_pFont = NULL;
	IsDrawing = FALSE;
}

CLPDView::~CLPDView()
{
	delete m_pFont;
}

BOOL CLPDView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CLPDView drawing

void CLPDView::OnDraw(CDC* pDC)
{
	IsDrawing = TRUE;

	pDC->SetMapMode(MM_LOENGLISH);
	CFont* pPreviousFont = pDC->SelectObject(GetFont());

	curCount = win.getlpdCt();
	curCyCt = win.getCurCCt();

	int header = 0;
	if (pDC->IsPrinting()) {
		PrintHeader(pDC);
		header = -60; 
	}

	if ((win.getminlog() != 0) && (win.getmaxlog() > win.getminlog())) {
		PrintVertScale(pDC, header);
	}

	int nFirst, nLast, nCount = curCount;

	if (!pBApp->globals.RunAvail) {
		ComputeVisibleLines(pDC, nFirst, nLast);
	}
	else {
		nFirst = 0;
		nLast = nCount - 1;
	}
	
	double xscale = 2.5 * 1.33;

	PrintHorzScale(pDC, nFirst, nLast, xscale, header);
	
	if (nCount == 1) {
		PrintOne(pDC, header);
	}
	else {
		Perfmeter(pDC, nFirst, nLast, xscale, header);
	}

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}

	if (!pBApp->globals.RunAvail) {
		HWND hWnd = GetSafeHwnd();
		int max = GetScrollLimit(SB_HORZ);
		::SetScrollPos(hWnd, SB_HORZ, max, TRUE);
	}

		IsDrawing = FALSE;
}
void CLPDView::PrintHeader(CDC * pDC)
{
	CString tmp;
	CGenericDoc* pDoc = GetDocument();
	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch
	double pi = logPixels / 100;

	double posx, mp, bot;
	bot = (1.0/pi);
	mp = (4.0/3.0)/bot;

	CFont* pPreviousFont = pDC->SelectObject(GetFont());
	pDC->SetMapMode(MM_LOENGLISH);

	for (int i = m_npage; i <= m_nMax; i++) { 
		tmp.Format("%s     -     Page %d ", (LPCSTR) pDoc->GetTitle(), i);
		posx = 20 + ((i-1) * (pageWd/mp) * (4.0/3.0));
 		pDC->TextOut(posx, -20, tmp);
	}

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}
}

void CLPDView::PrintOne(CDC * pDC, int header)
{
	CString tmp;
	int posy, posx;

	CRect rc;   
	pDC->GetClipBox(&rc);
	posx = rc.left;

	tmp.Format("%.0f", win.getminlog());
	posy = -1.33 * (100);
	pDC->TextOut(posx, posy + header, tmp);

	posx = 30.5 * 1.33;
	posy -= 6;
	pDC->MoveTo(posx, posy + header);
	pDC->LineTo(posx+2, posy + header);
}

void CLPDView::PrintVertScale(CDC * pDC, int header)
{
	CString tmp;
	double posx = 0, posy = 0, Range, unit, mark, step;

	CRect rc;   
	pDC->GetClipBox(&rc);
	posx = rc.left;

	Range = fabs(win.getmaxlog() - win.getminlog()) + 5;
	unit = Range / 100;
	mark = win.getminlog();
	step = unit * 20;	// 20% of the units at a time

	for (int i = 0; i <= 5; i++) {
 		tmp.Format("%.0f", mark);
		posy = -1.33 * (100 - (i*20));
		pDC->TextOut(posx, posy + header, tmp);
		mark += step;
	} 
}

void CLPDView::PrintHorzScale(CDC * pDC, int nFirst, int nLast,
							  double xscale, int header)
{
	CString tmp;
	double posx = 0, posy = header;
	int j, value;

	if(curCyCt) {
		posx = (28 * 1.33) + (nFirst * xscale);
		for (j = nFirst; j <= nLast; j++) {
			value = win.getCurCycle(j);
			if ((j % 8) == 0) {
				tmp.Format("|");
				pDC->TextOut(posx, posy-143, tmp);
				tmp.Format("%d", value);
				if ((j % 16) == 0) 
					pDC->TextOut(posx, posy-154, tmp);
				else
					pDC->TextOut(posx, posy-164, tmp);
			}
			posx += xscale;
		} // end for
	}
}

void CLPDView::Perfmeter(CDC* pDC, int nFirst, int nLast,
						 double xscale, int header)
{
	double posx, posy;
	double Range, diff, unit;
	int count = curCount;
	int value;

	if (count) { 
		CPen pen(PS_SOLID,1,RGB(0,160,0));
		CPen* oldPen = pDC->SelectObject(&pen);

	// Move the cursor to the first point on the graph
		posx = (30.5 * 1.33) + (nFirst * xscale);
	    
	// first find the basis for caluclations
		Range = fabs(win.getmaxlog() - win.getminlog()) + 1;
		unit = Range / 100;

		value = win.getlpdValues(nFirst);
		diff = fabs(value - win.getminlog());

		posy = ((100 - (diff / unit)) * -1.33) - 6;
		pDC->MoveTo(posx, posy + header);

	// Draw the lines on the screen
		for (int ct = nFirst+1; ct <= nLast; ct++){
			value = win.getlpdValues(ct);
			diff = fabs(value - win.getminlog());
			posy = ((100 - (diff / unit)) * -1.33) - 6;				
			posx += xscale;
			pDC->LineTo(posx, posy + header);
		}
	
		pDC->SelectObject(oldPen);
	}
}

void CLPDView::OnUpdate()
{
	ComputeViewMetrics();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());

	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CLPDView printing

BOOL CLPDView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CLPDView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch

	int numInches = (int)(m_DocSize.cx / 100);

	int numPages = (numInches * logPixels) / pageWd + 1;

	pInfo->SetMaxPage(numPages);

}

void CLPDView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CLPDView diagnostics

#ifdef _DEBUG
void CLPDView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CLPDView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CGenericDoc* CLPDView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGenericDoc)));
	return (CGenericDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLPDView message handlers

CFont *	CLPDView::GetFont()
{
	if(m_pFont == NULL) {
		m_pFont = new CFont;
		if(m_pFont) {
			//	Default to 10 pt Arial
			m_pFont->CreatePointFont(80, "Courier New");
		}
	}
	return m_pFont;
}

void CLPDView::ComputeViewMetrics()
{
	if(win.getlpdCt())
		m_DocSize.cx = (2.5 * 1.33) * (win.getlpdCt() + 1) + (30.5 * 1.33);
}

void CLPDView::ComputeVisibleLines(CDC* pDC, int& nFirst, int& nLast)
{
	int nCount = curCount;
 
	// Get the clipping region, in logical coordinates
	CRect rc;   
	pDC->GetClipBox(&rc);
	
	// Compute the first visible line
	if (rc.Width() < 150) {
		if (nCount <= 25)
			nFirst = 0; 
		else 
			nFirst = nCount - 25;
	}
	else nFirst = 0;
	
	// Compute the last visible line
	nLast =	(nCount ? nCount - 1 : 0);
}

void CLPDView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());
	ComputeViewMetrics();
	pBApp->globals.g_LpdHwnd = GetSafeHwnd();
}

LRESULT CLPDView::OnUserInvalLpd(WPARAM wParam, LPARAM lParam)
{
	if (!IsDrawing)
		OnUpdate();

	return 1;
}

LRESULT CLPDView::OnUserCloseLpd(WPARAM wParam, LPARAM lParam)
{
	CGenericDoc* pDoc = GetDocument();
	pDoc->OnCloseDocument();
	return 1;
}

void CLPDView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CLPDView::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CLPDView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CLPDView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CLPDView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
	OnUpdate();
}


void CLPDView::OnRunStop() 
{
	pBApp->globals.IsStop = TRUE;	
	pBApp->globals.RunAvail = TRUE;
	pBApp->globals.IsPause = FALSE;
}

void CLPDView::OnUpdateRunStop(CCmdUI* pCmdUI) 
{
		pCmdUI->Enable(!pBApp->globals.RunAvail);			
}

void CLPDView::OnRunPause() 
{
	pBApp->globals.IsPause = TRUE;	
}

void CLPDView::OnUpdateRunPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.IsPause && !pBApp->globals.RunAvail);		
}

void CLPDView::OnRunResume() 
{
	pBApp->globals.IsPause = FALSE;	
}

void CLPDView::OnUpdateRunResume(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.IsPause && !pBApp->globals.RunAvail);		
}

void CLPDView::OnRunRun() 
{
	pBApp->globals.g_pView->SendMessage(WM_USERCHILDRUN);	
}

void CLPDView::OnUpdateRunRun(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail && pBApp->globals.DocIsOpen);		
}

void CLPDView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if (pDC->IsPrinting()) {
		int pageWd = pDC->GetDeviceCaps(HORZRES);
		int originX = pageWd * (pInfo->m_nCurPage - 1);
		pDC->SetViewportOrg(-originX, 0);
		CView::OnPrepareDC(pDC, pInfo);
	}
	else {
		CScrollView::OnPrepareDC(pDC, pInfo);
	}
}

void CLPDView::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);			
}

void CLPDView::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);			
}

void CLPDView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	m_npage = pInfo->m_nCurPage;
	m_nMax = pInfo->GetMaxPage();

	CScrollView::OnPrint(pDC, pInfo);
}


void CLPDView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);
	
	OnUpdate();	
}

void CLPDView::OnCloseAllWins()
{
	CCloseWins printmssg;
	int ans = printmssg.DoModal();

	if(ans == IDOK)
		pBApp->globals.CloseAllWins();
}

void CLPDView::OnUpdateCloseAllWins(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}