// This class is for parsing the Parameter File and the Final Tree File
// so that the data can be redisplayed to the screen.
// Its primary use is with the "Display Output Windows" command.

#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "stdafx.h"
#include "Bambe.h"
#include "parfile.h"
#include "qsort.h"
#include "util.h"
#include "Window.h"

extern Windows win;
extern int my_get_categories(HWND, char**, int*); 
extern CBAMBEApp* pBApp;

/*
Current Parameter File Format (as of 9-4-99):
	Cycle  loglikelihood  Tree-Accept-Rate  Param-Accept-Rate  Radius  ValleyWin  Theta  Kappa  TTP  Gamma  PiA  PiG  PiC  PiT  (repeat Categories)
*/

ParamFile::ParamFile(char* file, char* treefile)
{
	strcpy(FileName, file);
	strcpy(TreeFile, treefile);

	logs = TAR = PAR = Radius = NULL;
	for (int i = 0; i < 10; i++)
		cat[i] = NULL;
	m_EstList = NULL;
}

ParamFile::~ParamFile()
{
	DeleteArrays();
}

/* function to find the mean and standard deviation for each column of numbers
   in a single pass through a file using the method of provisional means */
int ParamFile::FindMeans(int& num_col)
{
  char buf[MAXLINE+1],*s,*tok;
  double x;
  int n,i;
  num_col=0;

  ifstream infile;
  infile.open(FileName);

  /* read in first line */
  infile.get(buf, MAXLINE+1);
  if (infile.get() == '\n')
	strcat(buf, "\n");

  if(!infile.eof() && buf[0] != '\0') {	//if(fgets(buf,MAXLINE+1,stdin)) {
    s = buf;
    num_col=0;
    while(tok=strtok(s," \t\n")) {
      sscanf(tok,"%lf",&x);
      newmean[num_col] = oldmean[num_col] = x;
      sumsq[num_col++] = 0.;
      s = NULL;
    }
  }
  else {
	CString mssg;
	mssg.Format("Error: Cannot read first line of input");
	AfxMessageBox(mssg, MB_OK, 0);
    return 1;
  }
  n=1;

  /* read in rest of file */
  i=0;
  infile >> x;
  while(!infile.eof()) {
    if(i==0)
      n++;
    newmean[i] = oldmean[i] + (x - oldmean[i]) / (double) n;
    sumsq[i] += (x - oldmean[i]) * (x - newmean[i]);
    oldmean[i] = newmean[i];
    i = ((i+1) % num_col);
	infile >> x;
  }

  tot_lines = n;
  pBApp->globals.num_tags = (num_col - 6) / (4+NUM_BASES); 

  infile.close();
  return 0;
}

void ParamFile::InitializeArrays()
{
	DeleteArrays();		// Make sure there are no other arrays already allocated

	logs = new double[tot_lines + 1];
	TAR = new double[tot_lines + 1];
	PAR = new double[tot_lines + 1];
	Radius = new double[tot_lines + 1];
	for (int i = 0; i < pBApp->globals.num_tags; i++) {
		cat[i] = new param(tot_lines);
	}

	ifstream infile;
	infile.open(FileName);
	double x;
	int j;

	for (i = 0; i < tot_lines; i++) {
		infile >> x >> logs[i] >> TAR[i];
		infile >> PAR[i] >> Radius[i] >> x;
		for(j = 0; j < pBApp->globals.num_tags; j++) {
			infile >> cat[j]->theta[i] >> cat[j]->kap[i]
				>> cat[j]->ttp[i] >> cat[j]->gamma[i]  
				>> cat[j]->pia[i] >> cat[j]->pig[i] 
				>> cat[j]->pic[i] >> cat[j]->pit[i];
		}
	}
	infile.close();
}

void ParamFile::DeleteArrays()
{
	if (logs)
		delete [] logs;
	if (TAR)
		delete [] TAR;
	if (PAR)
		delete [] PAR;
	if (Radius)
		delete [] Radius;
	for (int i = 0; i < 10; i++)
		if (cat[i])
			delete cat[i];
}

void ParamFile::SortArrays()
{
	QSort(logs,0, tot_lines-1);
	QSort(TAR,0, tot_lines-1);
	QSort(PAR,0, tot_lines-1);
	QSort(Radius, 0, tot_lines-1);
	for (int i = 0; i < pBApp->globals.num_tags; i++) {
		QSort(cat[i]->theta,0, tot_lines-1);
		QSort(cat[i]->kap, 0, tot_lines-1);
		QSort(cat[i]->ttp, 0, tot_lines-1);
		QSort(cat[i]->gamma,0, tot_lines-1);
		QSort(cat[i]->pia, 0, tot_lines-1);
		QSort(cat[i]->pig,0, tot_lines-1);
		QSort(cat[i]->pic, 0, tot_lines-1);
		QSort(cat[i]->pit,0, tot_lines-1);
	}
}

void ParamFile::DetermineQuantiles()
{
	qInd[0] = (int)floor(0.01 * tot_lines) - 1;
	qInd[1] = (int)floor(0.05 * tot_lines) - 1;
	qInd[2] = (int)floor(0.10 * tot_lines) - 1;
	qInd[3] = (int)floor(0.25 * tot_lines) - 1;
	qInd[4] = (int)floor(0.50 * tot_lines) - 1;
	qInd[5] = (int)ceil(0.75 * tot_lines) - 1;
	qInd[6] = (int)ceil(0.90 * tot_lines) - 1;
	qInd[7] = (int)ceil(0.95 * tot_lines) - 1;
	qInd[8] = (int)ceil(0.99 * tot_lines) - 1;

	for (int i = 0; i < 9; i++ )
		qInd[i] = (qInd[i] >= 0 ? qInd[i] : 0);
}

void ParamFile::PrintSection(double *dat, int i, CString beg)
{
	CString str;
	double mean;
	int n = tot_lines;
	
	if (tot_lines %2 == 0) {
		mean = (dat[qInd[4]] + dat[qInd[4]+1]) / 2;
	}
	else mean = dat[qInd[4]];

	str.Format("%11.3lf +/- %6.3lf  %11.3lf %11.3lf %11.3lf %11.3lf %11.3lf %11.3lf %11.3lf %11.3lf %11.3lf", 
		newmean[i], sqrt(sumsq[i]/(double)(n-1)), dat[qInd[0]], dat[qInd[1]], dat[qInd[2]], dat[qInd[3]], mean, dat[qInd[5]], dat[qInd[6]], dat[qInd[7]], dat[qInd[8]]);  
	beg += str;
	m_EstList->AddTail(beg);
}

void ParamFile::PrintToScreen(int num_col)
{
	CString str, blank(" ");

	BOOL ok = pBApp->pMain->SendMessage(WM_USERNEWEST, 0,0);
	TRACE("Sending message WM_USERNEWEST %s\n", ok ? "true" : "false");

	str.Format("     Parameter         Mean      Standard     Quantiles: ");
	m_EstList->AddTail(str);
	str.Format("                                 Deviation     0.01        0.05        0.10        0.25        0.50        0.75        0.90        0.95        0.99");
	m_EstList->AddTail(str);
                          
	str.Format("___________________________________________________________________________________________________________________________________________________");
	m_EstList->AddTail(str);
	m_EstList->AddTail(blank);

	CString beg;

	beg.Format("Log Likelihood   ");
	PrintSection(logs, 1, beg);

	m_EstList->AddTail(blank);
	str.Format("Acceptance Rates:");
	m_EstList->AddTail(str);

	beg.Format("     Tree        ");
	PrintSection(TAR, 2, beg);

	beg.Format("     Parameter   ");
	PrintSection(PAR, 3, beg);

	m_EstList->AddTail(blank);
	beg.Format("Tree Radius      ");
	PrintSection(Radius, 4, beg);
	::PostMessage(pBApp->globals.g_ESTHwnd, WM_USERINVALEST, 0,0);

	int j = 6;
	for (int i = 0; i < pBApp->globals.num_tags; i++) {
		m_EstList->AddTail(blank);
		beg.Format("Category %d:",i+1);
		m_EstList->AddTail(beg);

		beg.Format("     Theta       ");
		PrintSection(cat[i]->theta, j++, beg);

		beg.Format("     Kappa       ");
		PrintSection(cat[i]->kap, j++, beg);

		beg.Format("     TTP         ");
		PrintSection(cat[i]->ttp, j++, beg);

		beg.Format("     Gamma       ");
		PrintSection(cat[i]->gamma, j++, beg);

		beg.Format("     Pi A        ");
		PrintSection(cat[i]->pia, j++, beg);

		beg.Format("     Pi G        ");
		PrintSection(cat[i]->pig, j++, beg);

		beg.Format("     Pi C        ");
		PrintSection(cat[i]->pic, j++, beg);

		beg.Format("     Pi T        ");
		PrintSection(cat[i]->pit, j++, beg);
	}

	::PostMessage(pBApp->globals.g_ESTHwnd, WM_USERINVALEST, 0,0);
}

void ParamFile::PrintToFile(char* output_file)
{
	ofstream outfile(output_file);

	CString strLine;
	char *line;
	for(POSITION pos = m_EstList->FindIndex(0); pos != NULL; ){
		strLine = m_EstList->GetNext(pos); 
		line = new char[strLine.GetLength() + 1];
		strcpy(line, strLine);
		TRACE("str %s\n", line);
		outfile << line << endl;
		delete line;
	}
	outfile.close();
}

void ParamFile::CalcEstimations(char* output_file, int Out)
{
	int num_col;

	HCURSOR newCursor, oldCursor;
  
	// Load the predefined Windows "hourglass" cursor.
	newCursor = AfxGetApp()->LoadStandardCursor(IDC_WAIT);	
	oldCursor = SetCursor(newCursor);		// set the new cursor

	if (pBApp->globals.EstList != NULL)
		delete pBApp->globals.EstList;

	pBApp->globals.EstList = new CStringList;
	m_EstList = pBApp->globals.EstList;

	if(FindMeans(num_col)) {
		//error
		return;
	}

	InitializeArrays();
	SortArrays();

	DetermineQuantiles();

	SetCursor(oldCursor);	// reset the old cursor

	PrintToScreen(num_col);

	if (Out) {
		PrintToFile(output_file);
	}
}

int ParamFile::ConvertToInt(char * buf)
{
	char num[50], *token = strchr(buf, '=');
	token = &(token[1]);
	strcpy(num, token);
	return atoi(num);
}

/* Reading in the *.par file to get the initial values - copy of MeanSD function*/
int ParamFile::GetValues(int tree_format, int param_wins, int& bad_tree_file) 
{
  char buf[MAXLINE+1],*s,*tok;
  double x;
  int num_col=0,n,i, cy;

  // read in parameter file
  if (param_wins) {
	ifstream infile;
	infile.open(FileName);
	pBApp->globals.burn = 0;			// Initialize burn cycle - just in case not given a value

	infile.get(buf, MAXLINE+1);
	if (infile.get() == '\n')
		strcat(buf, "\n");

	if(!infile.eof() && buf[0] != '\0') {	//if(fgets(buf,MAXLINE+1,stdin)) {
		s = buf;
		num_col=0;
		while(tok=strtok(s," \t\n")) {
			sscanf(tok,"%lf",&x);
			num_col++;
			s = NULL;
		}
	}
	else {
		CString mssg;
		mssg.Format("Error: Cannot read first line of input");
		AfxMessageBox(mssg, MB_OK, 0);
		return 0;
	}
	n=1;

	/* read in rest of file */
	i=0;
	infile >> x;
	cy = x;
	while(!infile.eof()) {
		for(i = 0; i < num_col; i++) {
			infile >> x;
		}
		n++;
		if(!infile.eof() && cy > x) {
			pBApp->globals.burn = cy;		// Determine the burn cycle - needed for the 2 different log-likelihood windows
		}
		cy = x;
	}

	infile.close();
	tot_lines = n;
	pBApp->globals.num_tags = (num_col - 6) / (4+NUM_BASES);
  }

  bad_tree_file = 0;
  // Read in tree file
  if(tree_format != -1) {
	bad_tree_file = win.ReadTree(TreeFile);
	TRACE("Read in the tree file\n");
	TRACE("bad_tree_file = %d\n", bad_tree_file);
  }

  return 1;
}

int ParamFile::GetMaxCycle()
{
	char buf[MAXLINE+1];
	int x, cy;

  // read in parameter file
	ifstream infile;
	infile.open(FileName);

	infile >> x;

	if(infile.eof()) {
		CString mssg;
		mssg.Format("Error: Cannot read first line of input");
		AfxMessageBox(mssg, MB_OK, 0);
		return 0;
	}

	while(!infile.eof()) {
		cy = x;
		infile.getline(buf, MAXLINE+1);
		infile >> x;
	}
	infile.close();
	return cy;
}

void ParamFile::SkipInfo(ifstream& infile)
{
	double x;
	infile >> x >> x;	// log, tree accept rate 
	infile >> x >> x;	// param. accept rate & tree radius
	infile >> x;		// valley win
	
	for (int i = 0; i < pBApp->globals.num_tags; i++) {
		infile >> x >> x >> x >> x;	// theta, kappa, ttp, gamma
		for(int b=0;b < NUM_BASES;b++)
			infile >> x;
	}
}

void ParamFile::Storage(ifstream& infile, int& cy)
{
	double x;

	win.CycleList(cy);

	infile >> x;
	win.AddToLpdList(x, cy);					// Log-likelihood

	infile >> x;
	win.AcceptRateList((char)(x*100));		// Tree acceptance Rate

	infile >> x;	
	win.KARateList((char)(x*100));			// Parameter acceptance Rate

	win.IncParamCounts();					// increment the number of parameters

	infile >> x;		// tree radius
	infile >> x;			// ValleyWin

	for (int i = 0; i < pBApp->globals.num_tags; i++) {
		infile >> x;
		win.ThetaList(i, x);
		infile >> x;
 		win.KappaList(i, x);
		infile >> x;		// ttp
		infile >> x;		// gamma

		for(int b=0;b < NUM_BASES;b++)
			infile >> x;
	}
}

int ParamFile::Parse(int begin, int end)
{
	int cy;
	ifstream infile;
	infile.open(FileName);
	TRACE("FileName %s\n", FileName);

	win.ParamFileInit(tot_lines);

	int flag = 0;

	if (begin >= 0) {
		if (end > 0) {	// Range: beginning/first cycle to ending cycle 
			infile >> cy;
			while(!infile.eof() && cy <= end) {
				TRACE("Cycle = %d ", cy);
				if (cy >= begin || flag) {
					flag = 1;
					TRACE("Storing...\n");
					Storage(infile, cy);
				}
				else  {
					TRACE("Skipping...\n");
					SkipInfo(infile);
				}
				infile >> cy;
			}
		}
		else {		// Range: beginning/first cycle to end of file
			infile >> cy;
			while(!infile.eof()) {
				if (cy >= begin || flag) {
					flag = 1;
					Storage(infile, cy);
				}
				else 
					SkipInfo(infile);
				infile >> cy;
			}
		}
	}
	else {		// Range: entire file
		infile >> cy;
		while(!infile.eof()) {
			Storage(infile, cy);
			infile >> cy;
		}
	}

	infile.close();
	return 1; 
}