// Summarize.cpp : implementation file
//

#include "stdafx.h"
#include "bambe.h"
#include "Summarize.h"
#include <fstream.h>
#include <process.h>
#include <stdlib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int errorCode;
extern void summarize(int, double, int, char**, int, ofstream&);
extern void Clear();

#define MAXLINE 1025
extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CSummarize dialog

static DWORD aSumHelpIDs[] = 
{
	IDC_TOP_FILE, HIDC_TOP_FILE,
	IDC_STATIC_TOP_FILE, HIDC_TOP_FILE,
	IDC_OUTPUT_FILE, HIDC_OUTPUT_FILE,
	IDC_STATIC_OUT_FILE2, HIDC_OUTPUT_FILE,
	IDC_BROWSE, HIDD_BROWSE,
	IDC_BROWSE2, HIDD_BROWSE,
	IDC_NUM_SKIP, HIDC_NUM_SKIP,
	IDC_STATIC_NUM_SKIP, HIDC_NUM_SKIP,
	IDC_THRESHOLD, HIDC_THRESHOLD,
	IDC_STATIC_THRESHOLD, HIDC_THRESHOLD,
	IDC_MAX_TOP, HIDC_MAX_TOP,
	IDC_STATIC_MAX_TOP, HIDC_MAX_TOP,
	IDOK, HIDD_OK,
	IDCANCEL, HIDD_DIALOG_CANCEL,
	0, 0
};

CSummarize::CSummarize(CWnd* pParent /*=NULL*/)
	: CDialog(CSummarize::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSummarize)
	m_max_top = 10;
	m_num_skip = 0;
	m_top_file = _T("");
	m_threshold = 0.8;
	m_output_file = _T("");
	//}}AFX_DATA_INIT

	tot_num_files = 0;

	if (pBApp->globals.sumList != NULL)
		delete pBApp->globals.sumList;

	pBApp->globals.sumList = new CStringList;
	m_sumList = pBApp->globals.sumList;

	// need to reinitialize these variables or else they keep incrementing
	Clear();
}

void CSummarize::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSummarize)
	DDX_Text(pDX, IDC_MAX_TOP, m_max_top);
	DDV_MinMaxInt(pDX, m_max_top, 2, 100);
	DDX_Text(pDX, IDC_NUM_SKIP, m_num_skip);
	DDX_Text(pDX, IDC_TOP_FILE, m_top_file);
	DDX_Text(pDX, IDC_THRESHOLD, m_threshold);
	DDV_MinMaxDouble(pDX, m_threshold, 0.5, 1.);
	DDX_Text(pDX, IDC_OUTPUT_FILE, m_output_file);
	DDV_MaxChars(pDX, m_output_file, 100);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSummarize, CDialog)
	//{{AFX_MSG_MAP(CSummarize)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(IDC_BROWSE2, OnBrowse2)
	ON_WM_CONTEXTMENU()
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSummarize message handlers
void CSummarize::GetTotFiles()
{
	int i;
	char c = '"', *temp, *str, *p2, *p3;

	temp = new char[m_top_file.GetLength()+1];
	strcpy(temp, m_top_file);

	if (m_top_file.IsEmpty()) {
		TRACE("string is empty\n");
		tot_num_files = 0;
	}
	else if (strchr(temp, c) == NULL) {
		TRACE("String has one file\n");
		tot_num_files = 1;
		m_file_list[0] = m_top_file;
	}
	else {
		i = 0;
		str = strchr(temp, c);
		str = &str[1];
		for(;;) {
			p2 = strchr(str, c);
			p3 = &p2[1];
			p2[0] = '\0';
			m_file_list[i].Format("%s", str);
			TRACE("file %d = %s\n", i, str);
			i++;
			str = p3;
			if(str[0] == '\0')
				break;
			else str = &str[2];
		}
		tot_num_files = i;
	}
	TRACE("Total number of files = %d\n", tot_num_files);

	delete temp;
}

void CSummarize::OnBrowse() 
{
	int i;
	char c = '"';
	CString seperator, quote(c);
	seperator.Format("%c %c", c, c);

	UpdateData();
	
	GetTotFiles();

	CFileDialog getFile(TRUE, NULL,"*.top",NULL,"Topology Files (*.top)|*.top|All Files (*.*)|*.*||");

	getFile.m_ofn.Flags |= OFN_ALLOWMULTISELECT;

	if (getFile.DoModal() == IDOK)
	{
		i = tot_num_files;
		for(POSITION p = getFile.GetStartPosition(); p; i++) {
			m_file_list[i] = getFile.GetNextPathName(p);
		}
		tot_num_files = i;

		if(tot_num_files == 1) {
			m_top_file = m_file_list[0];
		}
		else {
			m_top_file = quote;
			m_top_file += m_file_list[0];
			for (i = 1; i < tot_num_files; i++) {
				m_top_file += seperator;
				m_top_file += m_file_list[i];
			}
			m_top_file += quote;
		}

		if(m_output_file.IsEmpty()) {
			char *s = new char[m_file_list[0].GetLength() + 1];
			strcpy(s, m_file_list[0]);
			char *token = strchr(s, '.');
			strcpy(token, ".sum");
			m_output_file.Format("%s", s);
			delete [] s;
		}
	}

	UpdateData(FALSE);	
}

void CSummarize::OnOK() 
{
	UpdateData();

	int flag = 1;
	CString ErrMsg;

	TRACE("in OnOK tot_num_files = %d\n", tot_num_files);
	
	if(!tot_num_files) {  //m_top_file.IsEmpty()) {
		flag = 0;
		AfxFormatString1(ErrMsg, IDS_ERR_INVALIDFILE, m_top_file);
		MessageBox (ErrMsg);
	}
	else {
	for (int i = 0; i < tot_num_files; i++) {
		if (!pBApp->globals.IsValidFile(m_file_list[i])) {
			flag = 0;
			AfxFormatString1(ErrMsg, IDS_ERR_INVALIDFILE, m_file_list[i]);
			MessageBox (ErrMsg);
		}
	}
	}

	if(flag)
		CDialog::OnOK();
}


int CSummarize::RunSumMain(int skip, int cladetops, double threshold,
				char *names[100], int num_names, char* outputFile)
{
	TRACE("Beginning of RunSumMain \n");
  ofstream out;
  out.open(outputFile);
  if(out.fail()) {
	  CString ErrMsg;
	  AfxFormatString1(ErrMsg, IDS_ERR_NOTOPENFILE, outputFile);
	  MessageBox (ErrMsg);
	  TRACE("Exiting RunSumMain - couldn't open file\n");
	  return 1;
  }

  out << "******************** BAMBE Summarize Version 1 "
	 "********************\n\n";
  out << "Invocation: \n\tSkipped Lines = "<< skip << "\n\tThreshold = "
	  << threshold << "\n\tMax Clade Topology = " << cladetops;
  out << "\n\tFiles = ";
  for(int i=0;i<num_names;i++)
    out << names[i] << " ";
  out << "\n\n";

  errorCode = 0;
  TRACE("Before summarize\n");
  summarize(skip,threshold,cladetops,names,num_names, out);

  TRACE("after summarize errorCode = %d\n", errorCode);

  out.close();
  return errorCode;
}

void CSummarize::RunSummarize()
{
	char *names[100];
	char outputFile[100];

	HCURSOR newCursor, oldCursor;
  
	// Load the predefined Windows "hourglass" cursor.
	newCursor = AfxGetApp()->LoadStandardCursor(IDC_WAIT);	
	oldCursor = SetCursor(newCursor);		// set the new cursor

	TRACE("In summarize %s ", m_top_file);
	strcpy(outputFile, m_output_file);

	TRACE(" outfile %s \n", outputFile);

	TRACE("Total number of files: %d\n", tot_num_files);
	for (int i = 0; i < tot_num_files; i++) {
		char *c = new char[m_file_list[i].GetLength()+1];
		strcpy(c, m_file_list[i]);
		names[i] = c;
		TRACE("File %d: %s\n", i, names[i]);
	}	

	TRACE("Calling RunSumMain\n");
	int error = RunSumMain(m_num_skip, m_max_top, m_threshold, names, tot_num_files, outputFile);
	TRACE("Completed RunSumMain\n");

	for (i = 0; i < tot_num_files; i++) {
		delete names[i];
	}

	SetCursor(oldCursor);	// reset the old cursor

	if (error) {
		return;
	}
	else WriteToScreen(outputFile);
}

void CSummarize::WriteToScreen(char* outputFile)
{
	CString str;
	ifstream infile;
	infile.open(outputFile);

	BOOL ok = pBApp->pMain->SendMessage(WM_USERNEWSUM, 0,0);
	TRACE("Sending message WM_USERNEWSUM %s\n", ok ? "true" : "false");

	int i = 0;
	char buf[MAXLINE+1];
	infile.getline(buf, MAXLINE);

	str.Format("%s", buf);
	m_sumList->AddTail(str);
	str.Format("NOTE: The information from this window can be found in %s", outputFile);
	m_sumList->AddTail(str);

	infile.getline(buf, MAXLINE);

	while(!infile.eof()) {
		str.Format("%s", buf);
		m_sumList->AddTail(str);
	
		infile.getline(buf, MAXLINE);
		i++;
		if(i % 10 == 0) {
			::PostMessage(pBApp->globals.g_SUMHwnd, WM_USERINVALSUM, 0,0);
		}
	}

	::PostMessage(pBApp->globals.g_SUMHwnd, WM_USERINVALSUM, 0,0);
}

void CSummarize::OnBrowse2() 
{
	UpdateData();
	CFileDialog getFile(TRUE, NULL,"*.sum",NULL,"Summary Files (*.sum)|*.sum|All Files (*.*)|*.*||");

	if (getFile.DoModal() == IDOK) {
		m_output_file = getFile.GetPathName();
	}	
	UpdateData(FALSE);	
}

BOOL CSummarize::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSummarize::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	::WinHelp((HWND) *pWnd, AfxGetApp()->m_pszHelpFilePath,
		HELP_CONTEXTMENU, (DWORD)aSumHelpIDs);
}

BOOL CSummarize::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if (pHelpInfo->iContextType == HELPINFO_WINDOW){
	::WinHelp((HWND)pHelpInfo->hItemHandle, 
		AfxGetApp()->m_pszHelpFilePath,
		HELP_WM_HELP, (DWORD)aSumHelpIDs);
	}
	return TRUE;	
}
