// TuningParam.cpp : implementation file
//

#include "stdafx.h"
#include "bambe.h"
#include "TuningParam.h"
#include "property.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CTuningParam dialog
static DWORD aTuneHelpIDs[] = 
{
	IDC_DELTA, HIDD_DELTA,
	IDC_STATIC_DELTA, HIDD_DELTA,
	IDC_KAPPADELTA, HIDD_KAPPADELTA,
	IDC_STATIC_KD, HIDD_KAPPADELTA,
	IDC_THETA_CONST, HIDD_THETA_CONST,
	IDC_STATIC_TCONST,  HIDD_THETA_CONST,
	IDC_PI_CONST, HIDD_PI_CONST,
	IDC_STATIC_PCONST,  HIDD_PI_CONST,
	IDC_GAMMA_TUNE, HIDC_GAMMA_TUNE,
	IDC_TTP_TUNE, HIDC_TTP_TUNE,
	IDC_STATIC_GAMMA_TUNE, HIDC_GAMMA_TUNE,
	IDC_STATIC_TTP_TUNE, HIDC_TTP_TUNE,
	IDC_BACK, HIDD_DIALOG_BACK,
	IDC_OK, HIDD_DIALOG_OK,
	IDCANCEL, HIDD_DIALOG_CANCEL,
	0,0
};


CTuningParam::CTuningParam(CWnd* pParent /*=NULL*/)
	: CDialog(CTuningParam::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTuningParam)
	m_initial_kappa_delta = 0.1;
	m_initial_delta = 0.01;
	m_theta_const = 2000.0;
	m_pi_const = 2000.0;
	m_ttp_tune = 0.1;
	m_gamma_tune = 0.1;
	m_beta_tune = 10.0;
	//}}AFX_DATA_INIT
}


void CTuningParam::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTuningParam)
	DDX_Text(pDX, IDC_KAPPADELTA, m_initial_kappa_delta);
	DDX_Text(pDX, IDC_DELTA, m_initial_delta);
	DDX_Text(pDX, IDC_THETA_CONST, m_theta_const);
	DDX_Text(pDX, IDC_PI_CONST, m_pi_const);
	DDX_Text(pDX, IDC_TTP_TUNE, m_ttp_tune);
	DDX_Text(pDX, IDC_GAMMA_TUNE, m_gamma_tune);
	DDX_Text(pDX, IDC_LOCAL_TUNE, m_beta_tune);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTuningParam, CDialog)
	//{{AFX_MSG_MAP(CTuningParam)
	ON_BN_CLICKED(IDC_BACK, OnBack)
	ON_BN_CLICKED(IDC_OK, OnClickedOk)
	ON_WM_CONTEXTMENU()
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTuningParam message handlers

void CTuningParam::GetInfoTuningParam()
{
	pBApp->globals.valley_win = m_initial_delta;
	pBApp->globals.kappa_win = m_initial_kappa_delta;
	pBApp->globals.theta_const = m_theta_const; 
	pBApp->globals.pi_const = m_pi_const; 

	pBApp->globals.ttp_tune = m_ttp_tune;
	pBApp->globals.gamma_tune = m_gamma_tune;
	pBApp->globals.beta_tune = m_beta_tune;

	TRACE("initial delta %f\n", pBApp->globals.valley_win);
	TRACE("init kappa delta %f\n", pBApp->globals.kappa_win);
}


void CTuningParam::OnBack() 
{
	if (!UpdateData(TRUE))
	{
		TRACE0("UpdateData failed during dialog termination.\n");
		// the UpdateData routine will set focus to correct item
		return;
	}

	EndDialog(IDC_BACK);	
}

void CTuningParam::OnClickedOk() 
{
	UpdateData();

	CString ErrMsg;
	CString name;
	int flag = 1;
	
	if (m_initial_kappa_delta <= 0) {
		flag = 0;
		name = "Kappa Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}

	if (m_theta_const <= 0) {
		flag = 0;
		name = "Theta Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}

	if (m_pi_const <= 0) {
		flag = 0;
		name = "Pi Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}
	
	if (m_initial_delta <= 0) {
		flag = 0;
		name = "Global Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}

	if (m_gamma_tune <= 0) {
		flag = 0;
		name = "Gamma Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}

	if (m_ttp_tune <= 0) {
		flag = 0;
		name = "TTP Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}

	if (m_beta_tune <= 0) {
		flag = 0;
		name = "Local Beta Tuning Parameter";
		AfxFormatString1(ErrMsg, IDS_ERR_NONPOSITIVE, name);
		MessageBox (ErrMsg);
	}

	if (flag) {
		CDialog::OnOK();
	}
}

void CTuningParam::SetDefaultValues()
{
	m_initial_kappa_delta = 0.5;
	m_initial_delta = 0.01;
	m_theta_const = 2000.0;
	m_pi_const = 2000.0;
	m_ttp_tune = 0.5;
	m_gamma_tune = 0.5;
}

void CTuningParam::SetChangedValues()
{
	m_initial_kappa_delta = pBApp->globals.kappa_win;
	m_initial_delta = pBApp->globals.valley_win;
	m_theta_const = pBApp->globals.theta_const;
	m_pi_const = pBApp->globals.pi_const;
	m_ttp_tune = pBApp->globals.ttp_tune;
	m_gamma_tune = pBApp->globals.gamma_tune;
}

BOOL CTuningParam::OnInitDialog() 
{
	if (pBApp->globals.IsChange) 
		SetChangedValues();
//	else SetDefaultValues();

	CDialog::OnInitDialog();

	CWnd *pBox;

	if (pBApp->globals.burn_algorithm != 0 && 
		pBApp->globals.main_algorithm != 0) {
		pBox = GetDlgItem(IDC_DELTA);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_DELTA);
		if (pBox)
			pBox->EnableWindow(FALSE);
	}

	if (pBApp->globals.param_update_interval != 0) {

	if (pBApp->globals.update_kappa == 0 || pBApp->globals.model == 2) {
		pBox = GetDlgItem(IDC_KAPPADELTA);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_KD);
		if (pBox)
			pBox->EnableWindow(FALSE);
	}	

	if(pBApp->globals.model != 2) {
		pBox = GetDlgItem(IDC_STATIC_TNSPEC);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_GAMMA_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_TTP_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_GAMMA_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_TTP_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
	}

	if (pBApp->globals.update_ttp == 0) {
		pBox = GetDlgItem(IDC_TTP_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_TTP_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
	}
	if (pBApp->globals.update_gamma == 0) {
		pBox = GetDlgItem(IDC_GAMMA_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_GAMMA_TUNE);
		if (pBox)
			pBox->EnableWindow(FALSE);
	}

	if (pBApp->globals.update_pi == 0) {
		pBox = GetDlgItem(IDC_PI_CONST);
		if (pBox)
			pBox->EnableWindow(FALSE);
		pBox = GetDlgItem(IDC_STATIC_PCONST);
		if (pBox)
			pBox->EnableWindow(FALSE);
	}

	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTuningParam::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	::WinHelp((HWND) *pWnd, AfxGetApp()->m_pszHelpFilePath,
		HELP_CONTEXTMENU, (DWORD)aTuneHelpIDs);
}

BOOL CTuningParam::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if (pHelpInfo->iContextType == HELPINFO_WINDOW){
	::WinHelp((HWND)pHelpInfo->hItemHandle, 
		AfxGetApp()->m_pszHelpFilePath,
		HELP_WM_HELP, (DWORD)aTuneHelpIDs);
	}

	return CDialog::OnHelpInfo(pHelpInfo);
}

void CTuningParam::OnOK() 
{
// dummy function
}

