#include "stdafx.h"	
#include "BAMBE.h"
#include "FileRoot.h"

#include "Window.h"

#include "util.h"
#include "parameters.h"
#include "runsettings.h"
#include "basedata.h"
#include "sitedata.h"
#include "node.h"
#include "tree.h"

extern CBAMBEApp* pBApp;

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//CMemoryState oldMemState, newMemState, diffMemState;	// temp

Windows::Windows()
{
	outList = NULL;
	treList = NULL;
	NameList = NULL;
	curTree = NULL;
	pnodes = NULL;

	ToScr = KACt = THETACt = lpdCt = lpd2Ct = CurCCt = CurCCt2 = ARCt = KRCt = 0;
	ctk =ctgam= ctttp = ctT= 0;
	minlog = minlog2 = maxkap = maxtheta = maxttp = maxgam = 0.0;

	for(int k = 0; k < 10; k++) {
	   KappaArray[k] = NULL;
	   ThetaArray[k] = NULL;
	   TTPArray[k] = NULL;
	   GammaArray[k] = NULL;
	}

	lpdValues = NULL;
	lpd2Values = NULL;
	CurCycle = NULL;
	Cur2Cycle = NULL;
	AcceptRate = NULL;
	KapARate = NULL;

//    oldMemState.Checkpoint();		// Memory leak debugging purposes
}

Windows::~Windows()
{
	TRACE("In ~Windows: ");

	if (outList) {
		delete outList;		// CStringList destructor calls RemoveAll, which deletes the objects 
	}						// found in J:\DEVSTUDIO\VC\MFC\SRC\LIST_S.CPP
	if (treList) {
		delete treList;
	}
	if (NameList) {
		delete NameList;
	}

	TRACE("Deleted lists  ");

	if(pnodes) {
		Site(NumLeaves);
		delete pnodes;
	}

    ClearArrays();

	/*newMemState.Checkpoint();
    if(diffMemState.Difference( oldMemState, newMemState ) )
    {
        TRACE( "Memory leaked!\n" );
		diffMemState.DumpAllObjectsSince();
    }*/
}

void Windows::ClearArrays()
{
	for(int i = 0; i < 10; i++) {
	    if (KappaArray[i])
			delete [] KappaArray[i];
	    if (ThetaArray[i])
			delete [] ThetaArray[i];
	    if (TTPArray[i])
			delete [] TTPArray[i];
	    if (GammaArray[i])
			delete [] GammaArray[i];
	}
	if (lpdValues)
		delete [] lpdValues;
	if (lpd2Values)
		delete [] lpd2Values;
	if (CurCycle)
		delete [] CurCycle;
	if (Cur2Cycle )
		delete [] Cur2Cycle;
	if (AcceptRate)
		delete [] AcceptRate;
	if (KapARate)
		delete [] KapARate;

	for(int k = 0; k < 10; k++) {
	   KappaArray[k] = NULL;
	   ThetaArray[k] = NULL;
	   TTPArray[k] = NULL;
	   GammaArray[k] = NULL;
	}

	lpdValues = NULL;
	lpd2Values = NULL;
	CurCycle = NULL;
	Cur2Cycle = NULL;
	AcceptRate = NULL;
	KapARate = NULL;
}

int Windows::ThreadRun()
{
	return !(pBApp->globals.RunAvail);
}

void Windows::Initialize()
{
	ctk = ctT = minlog = minkap = minlog2 = maxkap = 0.0;
	ToScr = 1;

	if (outList) {
		delete outList;
	}
	if (treList) {
		delete treList;
	}
	if (NameList) {
		delete NameList;
	}

	outList = new CStringList; 
	treList = new CStringList;
	NameList = new CStringList;

	m_pMain = pBApp->pMain;

	doWindowsInit();
}

// added by HI - Put in the Windows class
void Windows::CheckMssg()
{
	if (pBApp->globals.IsPause) {
		pBApp->globals.PauseLoop();
	}

	if (pBApp->globals.IsStop) {
		 CString mssg;
		 mssg.Format("This option will stop program execution.");
		int ans = AfxMessageBox (mssg, MB_OKCANCEL, 0);
		if (ans == IDOK) {
			AfxEndThread(0);
		}
		else if (ans == IDCANCEL) {
			pBApp->globals.IsStop = FALSE;
			pBApp->globals.RunAvail = FALSE;
		}
	}
}

void Windows::doWindowsInit()
{
  ToScr = KACt = THETACt = lpdCt = lpd2Ct = CurCCt = CurCCt2 = ARCt = KRCt = 0;
  ctk = minlog = minlog2 = maxkap = maxtheta = 0.0;

  int tot_cycles = pBApp->globals.num_cycles + pBApp->globals.burn;

  ClearArrays();

  for(int k = 0; k < pBApp->globals.num_tags; k++) {
	  KappaArray[k] = new int [(tot_cycles / pBApp->globals.window_interval) + 1];
	  ThetaArray[k] = new int [(tot_cycles / pBApp->globals.window_interval) + 1];
	  TTPArray[k] = new int [(tot_cycles / pBApp->globals.window_interval) + 1];
	  GammaArray[k] = new int [(tot_cycles / pBApp->globals.window_interval) + 1];
  }
  
  lpdValues = new int[3*(pBApp->globals.burn / pBApp->globals.window_interval + 1)];
  if(2*pBApp->globals.burn <  pBApp->globals.num_cycles)
	lpd2Values = new int[((pBApp->globals.num_cycles - 2*pBApp->globals.burn) / pBApp->globals.window_interval + 1)];
  else 
	lpd2Values = new int[pBApp->globals.num_cycles / pBApp->globals.window_interval + 1];
  CurCycle = new int[(tot_cycles / pBApp->globals.window_interval) + 1];

  if(2*pBApp->globals.burn <  pBApp->globals.num_cycles)
	Cur2Cycle = new int[(pBApp->globals.num_cycles - 2*pBApp->globals.burn) / pBApp->globals.window_interval + 1];
  else
	  Cur2Cycle = new int[pBApp->globals.num_cycles / pBApp->globals.window_interval + 1];
  AcceptRate = new char[(tot_cycles / pBApp->globals.window_interval) + 1];
  KapARate = new char[(tot_cycles / pBApp->globals.window_interval) + 1];

  BOOL ok;
  // following added by HI to have the message sent to the screen for output
  if (pBApp->globals.ShowOutput) {
	  if(pBApp->globals.ShowTOP) {
  		ok = pBApp->pMain->SendMessage(WM_USERNEWTOP, 0,0);
		pBApp->globals.TopOpen = 1;		
		TRACE("Sending message WM_USERNEWTOP %s\n", ok ? "true" : "false");
	  }

	  if(pBApp->globals.ShowTAR) {		// Tree Acceptance Rate
		ok = pBApp->pMain->SendMessage(WM_USERNEWKAP, 0,0);
		pBApp->globals.KapOpen = 1;
		TRACE("Sending message WM_USERNEWKAP %s\n", ok ? "true" : "false");
	  }

	  if(pBApp->globals.ShowLPD2) {
		ok = pBApp->pMain->SendMessage(WM_USERNEWLPD2, 0,0);
		pBApp->globals.Lpd2Open = 1;
		TRACE("Sending message WM_USERNEWLPD2 %s\n", ok ? "true" : "false");
	  }

	  if(pBApp->globals.ShowLPD1) {
		ok = pBApp->pMain->SendMessage(WM_USERNEWLPD, 0,0);
		pBApp->globals.LpdOpen = 1;
		TRACE("Sending message WM_USERNEWLPD %s\n", ok ? "true" : "false");
	  }

	if (pBApp->globals.param_update_interval != 0) {
		if(pBApp->globals.ShowKAP && pBApp->globals.model != 2) {				// Kappa Values
			ok = pBApp->pMain->SendMessage(WM_USERNEWKAP2, 0,0);
			pBApp->globals.Kap2Open = 1;
			TRACE("Sending message WM_USERNEWKAP2 %s\n", ok ? "true" : "false");
		}

		if(pBApp->globals.ShowTheta) {
			ok = pBApp->pMain->SendMessage(WM_USERNEWTHETA, 0,0);
			pBApp->globals.THETAOpen = 1;
			TRACE("Sending message WM_USERNEWTHETA %s\n", ok ? "true" : "false");
		}

		if(pBApp->globals.ShowKAR) {				// Parameter Acceptance Rate
			ok = pBApp->pMain->SendMessage(WM_USERNEWKAR, 0,0);
			pBApp->globals.KarOpen = 1;
			TRACE("Sending message WM_USERNEWKAR %s\n", ok ? "true" : "false");
		}

		if(pBApp->globals.ShowTTP && pBApp->globals.model == 2) {				// TTP Values
			ok = pBApp->pMain->SendMessage(WM_USERNEWTTP, 0,0);
			pBApp->globals.TTPOpen = 1;
			TRACE("Sending message WM_USERNEWTTP %s\n", ok ? "true" : "false");
		}

		if(pBApp->globals.ShowGAM && pBApp->globals.model == 2) {				// Gamma Values
			ok = pBApp->pMain->SendMessage(WM_USERNEWGAM, 0,0);
			pBApp->globals.GAMOpen = 1;
			TRACE("Sending message WM_USERNEWGAM %s\n", ok ? "true" : "false");
		}
	}
  }

	pBApp->globals.IsBurn = 1;
}

void Windows::ParamFileInit(int tot_lines)
{
	KACt = THETACt = lpdCt = lpd2Ct = CurCCt = CurCCt2 = ARCt = KRCt = 0;
	minlog = minlog2 = 0; 
	maxkap = maxtheta = 0;

	pBApp->globals.burn = 1000;

	ClearArrays();

	for(int k = 0; k < pBApp->globals.num_tags; k++) {
		KappaArray[k] = new int [tot_lines + 1];
		ThetaArray[k] = new int [tot_lines + 1];
		TTPArray[k] = new int [tot_lines + 1];
		GammaArray[k] = new int [tot_lines + 1];
	}

	lpdValues = new int[tot_lines + 1];
	lpd2Values = new int[tot_lines + 1];
	CurCycle = new int[tot_lines + 1];
	Cur2Cycle = new int[tot_lines + 1];
	AcceptRate = new char[tot_lines + 1];
	KapARate = new char[tot_lines + 1];
}


// creates a tree structure
void Windows::CreateInitialTree(Tree *t, const BaseData& bd)
{
	if (pnodes) {
		delete pnodes;
	}

	NumLeaves = t->getNumLeaves();
	NumNodes= t->getNumNodes();
	Nodes.setSize(NumNodes);
	
	t->setNodes();
	vcopy(Nodes, t->getNodes(), NumNodes);

  	pnodes = new PrintNodes(NumLeaves, NumNodes, Nodes);
	curTree = (Internal*)(t->root->copyTopology(*pnodes));  
	if(pBApp->globals.TopOpen) {
		CString str;
		for (int i = 0; i < bd.getNumTaxa(); i++) {
			str.Format("%s", bd.getTaxaName(i));
			NameList->AddTail(str);
		}
		::PostMessage(pBApp->globals.g_TopHwnd, WM_USERINVALTOP, 0,0);
	}

	ToScr = 0;
}

void Windows::CopyTree(const Tree *t)
{
  if (ToScr && pBApp->globals.TopOpen) {
	  curTree = (Internal*)(t->root->copyTopology(*pnodes));

	  if (!pBApp->globals.mclock) { // From old code: || pBApp->globals.IsBurn) {
		curTree->splitBranch(pBApp->globals.outgroup-1);	  
	  }

	  curTree->setMinTaxa(); 
	::PostMessage(pBApp->globals.g_TopHwnd, WM_USERINVALTOP, 0,0);
  }
}

void Windows::OpenFile(char *filename, const char* extension, ofstream& f)
{
	char newRoot[82];
	TRACE("pBApp->globals.go = %d\n", pBApp->globals.go);
	if( (fopen(filename,"r") == NULL) || (pBApp->globals.go == 2))	// added by HI
		f.open(filename);
	else
	{
		pBApp->globals.go = GetNewRoot(newRoot);
		if (pBApp->globals.go == 1) {
			strcpy(pBApp->globals.file_root, newRoot);

		    OpenFile(filename, extension, f);
		}
		else if (pBApp->globals.go == 2) {
			f.open(filename);		
		}
	    else {
			pBApp->globals.RunAvail = TRUE;
			AfxEndThread(1);
		}
	}
}

void Windows::AddToNameList(char *name)
{
	CString strs;
	strs.Format("%s", name);
	NameList->AddTail(strs);
}

void Windows::CycleList(int i) 
{ 
	CurCycle[CurCCt++] = i; 
	if (i > (pBApp->globals.burn*2)) 
		Cur2Cycle[CurCCt2++] = i; 
}


// following function added by HI to add the loglikelihood to the list
void Windows::AddToLpdList(double log, int cycle)
{
	if (cycle <= (pBApp->globals.burn * 2)) {
		if (!lpdCt) { 
			maxlog = log;
			maxlog -= .5;
		}
		lpdValues[lpdCt++] = (int)log;

		if (log < minlog) {
			minlog = log;
			minlog += .5;
		}
		else if (log > maxlog) {
			maxlog = log;
			maxlog -= .5;
		}
	}
	else {
		if (!lpd2Ct) { 
			maxlog2 = log;
			maxlog2 -= .5;
		}
		lpd2Values[lpd2Ct++] = (int)log;

		if (log < minlog2) {
			minlog2 = log;
			minlog2 += .5;
		}
		else if (log > maxlog2) {
			maxlog2 = log;
			maxlog2 -= .5;
		}
		if (pBApp->globals.Lpd2Open) {
			TRACE("invalidating lpd2\n");
			::PostMessage(pBApp->globals.g_Lpd2Hwnd, WM_USERINVALLPD2, 0,0);
		}
	}

	if (pBApp->globals.LpdOpen) {
		::PostMessage(pBApp->globals.g_LpdHwnd, WM_USERINVALLPD, 0,0);	// always want to do this or else the scale is distorted
	}
}

void Windows::AddToParamRate(double ka, double i)
{
  double p = pBApp->globals.param_update_interval;
  char x =  char(100 * ((ka)*p/i));
  TRACE("adding to paramRate and invalidating\n");
  KapARate[KRCt++] = x;
  if (pBApp->globals.KarOpen) {
	::PostMessage(pBApp->globals.g_KarHwnd, WM_USERINVALKAR, 0,0);
  }
}

void Windows::AddToLists(int cycle, int accrt)
{
	// #3 in kap file added by HI
	CurCycle[CurCCt++] = cycle;
	if (cycle > (pBApp->globals.burn*2)) {
		Cur2Cycle[CurCCt2++] = cycle;
	}

	AcceptRate[ARCt++] = (accrt < 100 ? char(accrt): 100);
		
	if (pBApp->globals.KapOpen) {	// Tree Acceptance Rate
		::PostMessage(pBApp->globals.g_KapHwnd, WM_USERINVALKAP, 0,0);
	}

	if (pBApp->globals.param_update_interval != 0) {
		KACt++; // added by HI - to keep track of the number of kappa values
		THETACt++;
		TTPCt++;
		GAMCt++;
	}
}

void Windows::AddToGraphLists(int cycle, int accrt, double ka, 
						double interval, double ll)
{
	TRACE("ToScr = %d\n", ToScr);
	if (ToScr) {
		AddToLists(cycle, accrt);
		AddToParamRate(ka, interval);
		AddToLpdList(ll, cycle);
	}
}

void Windows::AddToParamArrays(double kap, double theta, double ttp, double gamma, int i)
{
 if (pBApp->globals.param_update_interval != 0 && ToScr) {
	 TRACE("adding to param arrays: %f, %f, %f, %f\n", kap, theta, ttp, gamma);

	if (ctk == 0)
		minkap = maxkap = kap;
	if (kap < minkap)
		minkap = kap;
	else if (kap > maxkap)
		maxkap = kap;

	KappaArray[i][KACt-1] = int(kap * 1000);
			
	if (pBApp->globals.Kap2Open) 
		::PostMessage(pBApp->globals.g_Kap2Hwnd, WM_USERINVALKAP2, 0,0);
	ctk++;

	if (ctT == 0)
		mintheta = maxtheta = theta;
	if (theta < mintheta)
		mintheta = theta;
	else if (theta > maxtheta)
		maxtheta = theta;

	ThetaArray[i][THETACt-1] = int(theta * 1000);
	
	if (pBApp->globals.THETAOpen) 
		::PostMessage(pBApp->globals.g_THETAHwnd, WM_USERINVALTHETA, 0,0);
	ctT++;
	
	if (ctttp == 0)
		minttp = maxttp = ttp;
	if (ttp < minttp)
		minttp = ttp;
	else if (ttp > maxttp)
		maxttp = ttp;

	TTPArray[i][TTPCt-1] = int(ttp * 1000);
			
	if (pBApp->globals.TTPOpen) 
		::PostMessage(pBApp->globals.g_TTPHwnd, WM_USERINVALTTP, 0,0);
	ctttp++;

	if (ctgam == 0)
		mingam = maxgam = gamma;
	if (gamma < mingam)
		mingam = gamma;
	else if (gamma > maxgam)
		maxgam = gamma;

	GammaArray[i][GAMCt-1] = int(gamma * 1000);

	if (pBApp->globals.GAMOpen) 	
		::PostMessage(pBApp->globals.g_GAMHwnd, WM_USERINVALGAM, 0,0);
	ctgam++;
 }
}

void Windows::KappaList(int i, double kap) 
{ 
	if (!i && !KACt)
		minkap = kap;
	if (kap < minkap)
		minkap = kap;
	else if (kap > maxkap)
		maxkap = kap;
	KappaArray[i][KACt] = (int)(kap * 1000); 
}

void Windows::ThetaList(int i, double theta) 
{ 
	if (!i && !THETACt)
		mintheta = theta;
	if (theta < mintheta)
		mintheta = theta;
	else if (theta > maxtheta)
		maxtheta = theta;
	ThetaArray[i][THETACt] = (int)(theta * 1000); 
}

// added by HI - update the windows after the burn algorithm
void Windows::updateBurnWindows()
{
    pBApp->globals.IsBurn = 0;

  	if (pBApp->globals.OutOpen)
	    ::PostMessage(pBApp->globals.g_OutHwnd, WM_USERINVALOUT, 0,0);
	if (pBApp->globals.LpdOpen)
	    ::PostMessage(pBApp->globals.g_LpdHwnd, WM_USERINVALLPD, 0,0);
	if (pBApp->globals.KapOpen)
		::PostMessage(pBApp->globals.g_KapHwnd, WM_USERINVALKAP, 0,0);
	if (pBApp->globals.THETAOpen)
		::PostMessage(pBApp->globals.g_THETAHwnd, WM_USERINVALTHETA, 0,0);

   ToScr = 0;
}

// added by HI - update the windows after the main algorithm is done
void Windows::updateMainWindows()
{
	if (pBApp->globals.Lpd2Open)
	  	::PostMessage(pBApp->globals.g_Lpd2Hwnd, WM_USERINVALLPD2, 0,0);
	if (pBApp->globals.KapOpen)
		::PostMessage(pBApp->globals.g_KapHwnd, WM_USERINVALKAP, 0,0);
	if (pBApp->globals.param_update_interval != 0) { 
		if (pBApp->globals.Kap2Open)
			::PostMessage(pBApp->globals.g_KapHwnd, WM_USERINVALKAP2, 0,0);
		if (pBApp->globals.KarOpen)
			::PostMessage(pBApp->globals.g_KarHwnd, WM_USERINVALKAR, 0,0);
		if (pBApp->globals.TTPOpen)
			::PostMessage(pBApp->globals.g_TTPHwnd, WM_USERINVALTTP, 0,0);
		if (pBApp->globals.GAMOpen)
			::PostMessage(pBApp->globals.g_GAMHwnd, WM_USERINVALGAM, 0,0);
	}

  if (pBApp->globals.ShowOutput) {
	BOOL ok = pBApp->pMain->SendMessage(WM_USERNEWTRE, 0,0);
	pBApp->globals.TreOpen = 1;
	TRACE("Sending message WM_USERNEWTRE %s %d\n", ok ? "true" : "false", pBApp->globals.TreOpen);
  }

  ToScr = 1;	
}

// The following was added by HI to print the .tre file to the screen.
// It uses the same method as described in the comment below.
void Windows::updateTreeWin(Tree *t)
{
    ToScr = 1;
    CopyTree(t);		// added by HI - copies the tree topology
	
	ofstream file("temp.txt");
	t->print(file);
	file.close();

	ifstream infile("temp.txt");
	int i = 0;
	char buf[267];
	CString str;

	infile.getline(buf, 267);

	str.Format("%s", buf);
	treList->AddTail(str);

	infile.getline(buf, 267);

	while(!infile.eof()) {
		str.Format("%s", buf);
		treList->AddTail(str);
	
		infile.getline(buf, 267);
		i++;
	}
	infile.close();

	if (pBApp->globals.TreOpen)	// update the final tree window after its final update
		::PostMessage(pBApp->globals.g_TreHwnd, WM_USERINVALTRE, 0,0);

	ToScr = 0;
	pBApp->globals.DidRun = 1;	
}

// The following was added by HI to print the .out file to the screen.
// The easiest way to accomplish this since specific functions are 
// called within each class is the write it all to a file and then close
// that file, reopen it, read it line-by-line and print to to the screen.
void Windows::PrintOutput(const RunSettings& rs, const SiteData& sd,
						  const Parameters& params)
{
  if (pBApp->globals.ShowOutput) {
	  /* Prints out summary information. */
	ofstream file("temp.txt");

	file << "Output generated by BAMBE version 2.01 Beta\n";
	file << "Data Summary" << endl << endl;
	sd.printInfo(file,rs);
	params.printInfo(file,rs,sd);
	sd.printMoreInfo(file);
	rs.print(file);

	file.close();
	ifstream infile("temp.txt");
 
	int i = 0;
	char buf[267];
	CString str;
	infile.getline(buf, 267);

	str.Format("%s", buf);
	outList->AddTail(str);

	infile.getline(buf, 267);

	while(!infile.eof()) {
		str.Format("%s", buf);
		outList->AddTail(str);
	
		infile.getline(buf, 267);
		i++;
	}
	infile.close();

	BOOL t = pBApp->pMain->SendMessage(WM_USERNEWOUT, 0,0);
  	pBApp->globals.OutOpen = 1;
	TRACE("Sending message WM_USERNEWOUT %s", t ? "true" : "false");
  }
}

int Windows::ReadTree(char *filename)
{
	// Test the final tree file to see if it is in Newick Format - if not, quit
	ifstream file(filename);
    int c;

	while(isspace(c=file.get()))
		;
    if(c != '(') {
		CString error;
		error.Format("\nThe tree file specified is not in Newick format. The final tree topology will not be displayed.\n");
		AfxMessageBox(error, MB_OK, 0);
		TRACE("Bad tree file, returning\n");
		return 1;
	}
	file.close();

	// Ok - ready to create the tree
	if (NameList) {
		NameList->RemoveAll();
		delete NameList;
	}
	NameList = new CStringList;

	BaseData bd(filename);
	Site(bd.getNumTaxa());	        // Set the length of all sites.
	SiteData sd(bd);

	CString str;
	for (int i = 0; i < bd.getNumTaxa(); i++) {
		str.Format("%s", bd.getTaxaName(i));
		NameList->AddTail(str);
	}

	Tree* t = new Tree(sd,filename);
	NumLeaves = t->getNumLeaves();
	NumNodes= t->getNumNodes();

	Vector<Node*> tempNodes(NumNodes);
	tempNodes = t->getNodes();

	// Actually need to COPY the nodes themselves because they disappear! (deleted)
	Nodes.setSize(NumNodes);
	for(int i1=0;i1<NumLeaves;i1++) 
		Nodes[i1] = new Leaf(tempNodes[i1]);
	for(int i2=NumLeaves;i2<NumNodes;i2++)
		Nodes[i2] = new Internal(tempNodes[i2]);

	if(pnodes) {
		delete pnodes;
	}
  	pnodes = new PrintNodes(NumLeaves, NumNodes, Nodes);
	
	curTree = (Internal*)(t->root->copyTopology(*pnodes));	//((SimpleNode*)t->root)->copyTree();

	delete t;
	return 0;
}


// Old CopyTree
/*//		g_cs.Lock();
		curTree->deleteTree();
		curTree = ((SimpleNode*)t->root)->copyTree();
		curTree->setMinTaxa(); 
//		g_cs.Unlock();
	  }
	  else {	// Non-clock tree
//		g_cs.Lock();
		curTree->deleteTree();		// deleted ->root
		curTree= ((SimpleNode*)t->root)->copyTree();
		curTree->splitBranch(pBApp->globals.outgroup-1);
		curTree->setMinTaxa(); 
//		g_cs.Unlock();*/
