#ifndef WINDOWSCLASS
#define WINDOWSCLASS

#include <fstream.h>
#include "stdafx.h"

#include "util.h"
#include "parameters.h"
#include "runsettings.h"
#include "site.h"
#include "basedata.h"
#include "sitedata.h"
#include "tree.h"

class Windows {
private:
	// lists of data for the output windows (graphing windows)
	int *lpdValues, *lpd2Values, *CurCycle, *Cur2Cycle;
	int *KappaArray[MAX_CATEGORIES], *ThetaArray[MAX_CATEGORIES];
	int *TTPArray[MAX_CATEGORIES], *GammaArray[MAX_CATEGORIES];
	char *AcceptRate, *KapARate;

	// counters - keeps track of the current position in the array of the above lists
	int lpdCt, lpd2Ct, CurCCt, CurCCt2, ARCt, KRCt, KACt, THETACt, TTPCt, GAMCt;

	// values for creation of the likelihood and kappa graphs
	double minlog, maxlog, maxkap, minkap, minlog2, maxlog2;
	double mintheta, maxtheta, minttp, maxttp, mingam, maxgam;
	int ctk, ctT, ctttp, ctgam; 

	// ToScr indicates if the data is to be printed to the screen or not
	int ToScr;

	// pointer to the main window
	CWnd* m_pMain;

	void ClearArrays();

public:
	// lists of data for the output windows (strings)
	CStringList *outList, *treList, *NameList;

	//Current tree topology
	Internal *curTree;	
	int NumLeaves;		// Number of leaves in the tree
	int NumNodes;
	Vector<Node*> Nodes;
	PrintNodes* pnodes;
	
	Windows();
	~Windows();

	void Initialize();
	void CheckMssg();
	void PrintOutput(const RunSettings&, const SiteData&, const Parameters&);
	void OpenFile(char*, const char*, ofstream&);
	void CreateInitialTree(Tree*, const BaseData&);
	void CopyTree(const Tree*);
	int  ThreadRun();

	void doWindowsInit();
	void updateBurnWindows();
	void updateMainWindows();
	void updateTreeWin(Tree*);
	void updateToScr(int x) { ToScr = x; }

	void ParamFileInit(int);
	int  ReadTree(char*, int);

	void AddToOutList(CString str) { outList->AddTail(str); }
	void AddToTreList(CString str) { treList->AddTail(str); }
	void AddToNameList(char*);

	void AddToLpdList(double, int);
	void AddToLists(int, int);
	void AddToParamRate(double, double);
	void AddToGraphLists(int, int, double, double, double);

	void AddToParamArrays(double, double, double, double, int);

	void CycleList(int);
	void AcceptRateList(char i) { AcceptRate[ARCt++] = i; }
	void KARateList(char i) { KapARate[KRCt++] = i; }
	void KappaList(int, double);
	void ThetaList(int, double);
	void IncParamCounts() {KACt++; THETACt++;}
	int getToScr() { return ToScr; }

	int getlpdCt() { return lpdCt; }
	int getlpd2Ct() { return lpd2Ct; }
	int getCurCCt() { return CurCCt; }
	int getCurCCt2() { return CurCCt2; }
	int getARCt() { return ARCt; }
	int getKRCt() { return KRCt; }
	int getKACt() { return KACt; }
	int getTHETACt() { return THETACt; }
	int getctk() { return ctk; }
	int getGAMCt() { return GAMCt; }
	int getTTPCt() { return TTPCt; }

	int getOutCt() { return outList->GetCount(); }
	int getTreCt() { return treList->GetCount(); }

	int getlpdValues(int i) { return lpdValues[i]; }
	int getlpd2Values(int i) { return lpd2Values[i]; }
	int getCurCycle(int i) { return CurCycle[i]; }
	int getCur2Cycle(int i) { return Cur2Cycle[i]; }
	char getAcceptRate(int i) { return AcceptRate[i]; }
	char getKapARate(int i) { return KapARate[i]; }
	int getKappaArray(int i1, int i2) { return KappaArray[i1][i2]; }
	int getThetaArray(int i1, int i2) { return ThetaArray[i1][i2]; }
	int getGammaArray(int i1, int i2) { return GammaArray[i1][i2]; }
	int getTTPArray(int i1, int i2) { return TTPArray[i1][i2]; }
	
	double getminlog() { return minlog; }
	double getmaxlog() { return maxlog; }
	double getminkap() { return minkap; }
	double getmaxkap() { return maxkap; }
	double getminlog2() { return minlog2; }
	double getmaxlog2() { return maxlog2; }
	double getmintheta() { return mintheta; }
	double getmaxtheta() { return maxtheta; }
	double getminTTP() { return minttp; }
	double getmaxTTP() { return maxttp; }
	double getmingam() { return mingam; }
	double getmaxgam() { return maxgam; }

	void ClearOutList() { if (outList->GetCount() != 0) outList->RemoveAll(); }
	void ClearTreList() { if (treList->GetCount() != 0) treList->RemoveAll(); }
	void ClearNameList(){ if (NameList->GetCount() != 0) NameList->RemoveAll(); }

	CWnd* getMain() { return m_pMain; }
};

#endif
