//Tree Acceptance Rate
// KAPView.cpp : implementation of the CKAPView class
//

#include "stdafx.h"
#include "BAMBE.h"
#include "Window.h"

//#include "KAPDoc.h"
#include "GenDoc.h"
#include "KAPView.h"
#include "LogScrollView.h"
#include "CloseWins.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern Windows win;
extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CKAPView

IMPLEMENT_DYNCREATE(CKAPView, CScrollView)

BEGIN_MESSAGE_MAP(CKAPView, CScrollView)
	//{{AFX_MSG_MAP(CKAPView)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_WM_HSCROLL()
	ON_COMMAND(ID_RUN_STOP, OnRunStop)
	ON_UPDATE_COMMAND_UI(ID_RUN_STOP, OnUpdateRunStop)
	ON_COMMAND(ID_RUN_PAUSE, OnRunPause)
	ON_UPDATE_COMMAND_UI(ID_RUN_PAUSE, OnUpdateRunPause)
	ON_COMMAND(ID_RUN_RESUME, OnRunResume)
	ON_UPDATE_COMMAND_UI(ID_RUN_RESUME, OnUpdateRunResume)
	ON_COMMAND(ID_RUN_RUN, OnRunRun)
	ON_UPDATE_COMMAND_UI(ID_RUN_RUN, OnUpdateRunRun)
	ON_COMMAND(ID_CLOSEALLWINS, OnCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_CLOSEALLWINS, OnUpdateCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_USERINVALKAP, OnUserInvalKap)
	ON_MESSAGE(WM_USERCLOSEKAP, OnUserCloseKap)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKAPView construction/destruction

CKAPView::CKAPView() : m_DocSize(100, 180)
{
	acceptCt = curCyCt = 0;
	m_pFont = NULL;
	IsDrawing = FALSE;
}

CKAPView::~CKAPView()
{
	delete m_pFont;
}

BOOL CKAPView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CKAPView drawing

void CKAPView::OnDraw(CDC* pDC)
{
	IsDrawing = TRUE;

	pDC->SetMapMode(MM_LOENGLISH);
	CFont* pPreviousFont = pDC->SelectObject(GetFont());

	acceptCt = win.getARCt();
	curCyCt = win.getCurCCt();

	int header = 0;
	if (pDC->IsPrinting()) {
		PrintHeader(pDC);
		header = -60; 
	}

	PrintVertScale(pDC, header);

	int nFirst, nLast, ok;
	if (!pBApp->globals.RunAvail) {
		ok = ComputeVisibleLines(pDC, nFirst, nLast);
		if(!ok) {
			nFirst = 0;
			nLast = 0;
		}
	}
	else {
		nFirst = 0;
		nLast = acceptCt - 1;
	}

	double xscale = 2.5 * 1.33;
	PrintHorzScale(pDC, nFirst, nLast, xscale, header);

	Perfmeter(pDC, nFirst, nLast, xscale, header);

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}

	if (!pBApp->globals.RunAvail) {
		HWND hWnd = GetSafeHwnd();
		int max = GetScrollLimit(SB_HORZ);
		::SetScrollPos(hWnd, SB_HORZ, max, TRUE);
	}

	IsDrawing = FALSE;
}

void CKAPView::PrintHeader(CDC * pDC)
{
	CString tmp;
	CGenericDoc* pDoc = GetDocument();
	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch
	double pi = logPixels / 100;

	double posx, mp, bot;
	bot = (1.0/pi);
	mp = (4.0/3.0)/bot;

	TRACE("pix per inch = %d, pi %f, bot = %f,mp = %f\n", logPixels, pi, bot, mp);

	CFont* pPreviousFont = pDC->SelectObject(GetFont());
	pDC->SetMapMode(MM_LOENGLISH);

	for (int i = m_npage; i <= m_nMax; i++) { 
		tmp.Format("%s     -     Page %d ", (LPCSTR) pDoc->GetTitle(), i);
		posx = 20 + ((i-1) * (pageWd/mp) * (4.0/3.0));
 		pDC->TextOut(posx, -20, tmp);
	}

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}
}

void CKAPView::PrintVertScale(CDC * pDC, int header)
{
	CString tmp;
	double posx = 0, posy = 0;

	CRect rc;   
	pDC->GetClipBox(&rc);
	posx = rc.left;

	for (int i = 10; i >= 0; i--) {
		tmp.Format("%d", i * 10);
		posy = -1.33 * (100 - (i*10));
		pDC->TextOut(posx, posy + header, tmp);
	}
}

void CKAPView::PrintHorzScale(CDC * pDC, int nFirst, int nLast,
							  double xscale, int header)
{
	CString tmp;
	double posx = 0, posy = header;
	int j, value;

	if(curCyCt) {
		posx = (10 * 1.33) + (nFirst * xscale);
		for (j = nFirst; j <= nLast; j++) {
			value = win.getCurCycle(j);
			if ((j % 8) == 0) {
				tmp.Format("|");
				pDC->TextOut(posx, posy - 143, tmp);
				tmp.Format("%d", value);
				if ((j % 16) == 0) 
					pDC->TextOut(posx, posy - 154, tmp);	// was -150
				else
					pDC->TextOut(posx, posy - 164, tmp);	// was -158
			}
			posx += xscale;
		} // end for
	}
}

void CKAPView::Perfmeter(CDC* pDC, int nFirst, int nLast, 
						 double xscale, int header)
{
	double posx,  posy;
	int value;

	if (acceptCt) {
		CPen pen(PS_SOLID,1,RGB(0,0,160));
		CPen* oldPen = pDC->SelectObject(&pen);

	// Move the cursor to the first point on the graph
		posx = (12.5 * 1.33) + (nFirst * xscale);

		value = win.getAcceptRate(nFirst);
		posy = ((100 - value) * -1.33) - 6;
		pDC->MoveTo(posx, posy + header);

	// Draw the lines on the screen
		for (int ct = nFirst+1; ct <= nLast; ct++){
			value = win.getAcceptRate(ct);
			posy = ((100 - value) * -1.33) - 6;
			posx += xscale;
			pDC->LineTo(posx, posy + header);
		}
		pDC->SelectObject(oldPen);
	}
}

void CKAPView::OnUpdate()
{
	ComputeViewMetrics();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());

	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CKAPView printing

BOOL CKAPView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

 void CKAPView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch
	TRACE("logpixels = %d\n", logPixels);

	int numInches = (int)(m_DocSize.cx / 100);

	int numPages = (numInches * logPixels) / pageWd + 1;

	pInfo->SetMaxPage(numPages);
}

void CKAPView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CKAPView diagnostics

#ifdef _DEBUG
void CKAPView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CKAPView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CGenericDoc* CKAPView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGenericDoc)));
	return (CGenericDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CKAPView message handlers

CFont *	CKAPView::GetFont()
{
	if(m_pFont == NULL) {
		m_pFont = new CFont;
		if(m_pFont) {
			//	Default to 10 pt Arial //was 100
			m_pFont->CreatePointFont(80, "Courier New");
		}
	}
	return m_pFont;
}

void CKAPView::ComputeViewMetrics()
{
	if(win.getARCt())
		m_DocSize.cx = (2.5 * 1.33) * win.getARCt() + (22 * 1.33);
}

int CKAPView::ComputeVisibleLines(CDC* pDC, int& nFirst, int& nLast)
{
	if (!acceptCt)
		return 0;

	int nCount = acceptCt;
    
		// Get the clipping region, in logical coordinates
	CRect rc;   
	pDC->GetClipBox(&rc);
	
		// Compute the first visible line
	if (rc.Width() < 150) {
		if (nCount <= 25)
			nFirst = 0; 
		else 
			nFirst = nCount - 25;
	}
	else nFirst = 0;

	// Compute the last visible line
	nLast =	nCount - 1;
	
	return 1;
}

void CKAPView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());
	ComputeViewMetrics();
	pBApp->globals.g_KapHwnd = GetSafeHwnd();
}

LRESULT CKAPView::OnUserInvalKap(WPARAM wParam, LPARAM lParam)
{
	if (!IsDrawing) {
		OnUpdate();
	}

	return 1;
}

LRESULT CKAPView::OnUserCloseKap(WPARAM wParam, LPARAM lParam)
{
	CGenericDoc* pDoc = GetDocument();
	pDoc->OnCloseDocument();	
	return 1;
}
void CKAPView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CKAPView::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

void CKAPView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);		
}

void CKAPView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);		
}

void CKAPView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
	OnUpdate();
}

void CKAPView::OnRunStop() 
{
	pBApp->globals.IsStop = TRUE;
	pBApp->globals.RunAvail = TRUE;
	pBApp->globals.IsPause = FALSE;
}

void CKAPView::OnUpdateRunStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.RunAvail);			
}

void CKAPView::OnRunPause() 
{
	pBApp->globals.IsPause = TRUE;	
}

void CKAPView::OnUpdateRunPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.IsPause && !pBApp->globals.RunAvail);			
}

void CKAPView::OnRunResume() 
{
	pBApp->globals.IsPause = FALSE;	
}

void CKAPView::OnUpdateRunResume(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.IsPause && !pBApp->globals.RunAvail);		
}

void CKAPView::OnRunRun() 
{
	pBApp->globals.g_pView->SendMessage(WM_USERCHILDRUN);		
}

void CKAPView::OnUpdateRunRun(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail && pBApp->globals.DocIsOpen);			
}

void CKAPView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if (pDC->IsPrinting()) {
		int pageWd = pDC->GetDeviceCaps(HORZRES);
		int pageHt = pDC->GetDeviceCaps(VERTRES);
		POINT cr;
		cr.x = pageWd;
		cr.y = pageHt;
		pDC->DPtoLP(&cr);
		TRACE("HORZRES = %d, DPtoLP %d\n", pageWd, cr.x);
		TRACE("VERTRES = %d, DPtoLP %d\n", pageHt, cr.y);
		int originX = (pageWd) * (pInfo->m_nCurPage - 1);
		pDC->SetViewportOrg(-originX, 0);
		CView::OnPrepareDC(pDC, pInfo);
	}
	else {
		CScrollView::OnPrepareDC(pDC, pInfo);
	}
}

void CKAPView::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);		
}

void CKAPView::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);		
}

void CKAPView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	m_npage = pInfo->m_nCurPage;
	m_nMax = pInfo->GetMaxPage();

	CScrollView::OnPrint(pDC, pInfo);
}

void CKAPView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);

	OnUpdate();
}

void CKAPView::OnCloseAllWins()
{
	CCloseWins printmssg;
	int ans = printmssg.DoModal();

	if(ans == IDOK)
		pBApp->globals.CloseAllWins();
}

void CKAPView::OnUpdateCloseAllWins(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}
