#define VERSION 7

/* Summarize for BAMBE, Version 7.
 * (c) Copyright 1998, 1999 by Donald Simon & Bret Larget
 * Department of Mathematics/Computer Science, Duquesne University
 * 10/21/99
 */

#include "stdafx.h"	// added by HI

CString mssg;
extern int errorCode;
void PrintMssg() 
{
	CString exit("\nAborting Summary.");
	mssg += exit;
	AfxMessageBox (mssg, MB_OK, 0);	
	errorCode = 1;
}
// Changed error message printing to Windows standard
// All printf were changed to fprintf(out, so that output can be sent to a file


#include <iostream.h>
#include <fstream.h>

#define WINDOWS

#ifdef WINDOWS
#include <strstrea.h>
#else
#include <strstream.h>
#endif
#include <iomanip.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define DEFAULTTHRESHOLD .80
#define MAXCLADETOPS 100

#define SSIZE 200003 /* Must be larger than MAXTREES */
#define CSIZE 10709 /* Must be larger than MAXCLADES */
#define MAXLEAVES 200
#define MAXLINE 6*MAXLEAVES

#define SETSIZE ((MAXLEAVES+30)/31)
#define MAXCLADES 5000
#define MAXTREES 15000
const unsigned int bit = (1<<30);

#define SHASH(k1,k2)	((((k1)<<6)+(k2)) % SSIZE)

void usageError(char *name)
{
  cerr << "Usage: " << name << " [-n skipped_lines] [-p threshold] "
       << "[-c maximum_clade_topologies] <file 1> <file 2> ..." << endl;
  exit(1);
}

typedef unsigned int Set[SETSIZE];

unsigned int chash(const Set s)	
{
  unsigned int n;

  n = s[0];
  for(int i=1;i<SETSIZE;i++)
    n = ((n << 6) + s[i]) % CSIZE;
  return n;
}

void makeEmptySet(Set s)
{
  for(int i=0;i<SETSIZE;i++)
    s[i] = 0;
}

void copySet(Set s1, const Set s2)
{
  for(int i=0;i<SETSIZE;i++)
    s1[i] = s2[i];
}

void setUnion(const Set s1, const Set s2, Set s)
{
  for(int i=0;i<SETSIZE;i++)
    s[i] = s1[i] | s2[i];
}

void setDifference(const Set s1, const Set s2, Set s)
{
  for(int i=0;i<SETSIZE;i++)
    s[i] = s1[i] & ~s2[i];
}

int setEqual(const Set s1, const Set s2)
{
  for(int i=0;i<SETSIZE;i++)
    if(s1[i] != s2[i])
      return 0;
  return 1;
}

int subset (const Set s1, const Set s2)
{
  for(int i=0;i<SETSIZE;i++)
    if(s1[i] & ~s2[i])
      return 0;
  return 1;
}

void singleton(const int n, Set s)
{
  for(int i=0;i<SETSIZE;i++)
    s[i] = 0;
  s[(n-1)/31] = (1 << (30 - ((n-1) % 31)));
}

void printSet(const Set s, ofstream& out)
{
  int first = 1;
  unsigned int n;

  out << "{";
  for(int i=0;i<SETSIZE;i++) {
    n = s[i];
    for(int j=1;j<32;j++,n<<=1)
      if(n & bit) {
	if(first)
	  first = 0;
	else 
	  out << ",";
	out << j+31*i;
      }
  }
  out << "}";
}

void printElements(const Set s, ofstream& out)
{
  unsigned int n;

  for(int i=0;i<SETSIZE;i++) {
    n = s[i];
    for(int j=1;j<32;j++,n<<=1)
    if(n & bit)
      out << setw(12) << " " << setw(3) << j+31*i << endl;
  }
}

int cmpSet(const Set s1, const Set s2)
{
  int c;

  for(int i=0;i<SETSIZE;i++)
    if(c=(s1[i]-s2[i]))
      return c;
  return 0;
}

int cindex[CSIZE], sindex[SSIZE];
int num_clades = 0, num_trees = 0;

class Ctab {
 public:
  Set set;
  int count;
  int trees;
  int first;
  int name;
  int time;
  int lasttopnum;
  int trans[MAXCLADETOPS+1][MAXCLADETOPS+1];
} ctab[MAXCLADES];

class Stab {
 public:
  int left;
  int right;
  int count;
  int clade;
  int next;
  int name;
  int num;
} stab[MAXTREES];

void Clear()	// added by HI - to clear array for multiple runs
{
	int i;
	for(i = 0; i < CSIZE; i++) {
		cindex[i] = sindex[i] = 0;
	}
	for(i = 0; i < MAXTREES; i++) {
		stab[i].left = stab[i].right = stab[i].next = -1;
	}
	num_clades = num_trees = 0;
}

void printTree(int k,int printId, ofstream& out)
{
  if(k<0)
    out << -k;
  else if(stab[k].name) {
    out << char(ctab[stab[k].clade].name+'A'-1);
    if(printId)
      out << stab[k].name;
  }
  else {
    out << "(";
    printTree(stab[k].left,printId,out);
    out << ",";
    printTree(stab[k].right,printId,out);
    out << ")";
  }
}

void addTrans(int c, int t, int time, int cladetops)
{
  if(ctab[c].trees<=cladetops) {
    if(ctab[c].time == time-1)
      ctab[c].trans[ctab[c].lasttopnum][stab[t].num]++;
    else {
      ctab[c].trans[ctab[c].lasttopnum][0]++;
      ctab[c].trans[0][stab[t].num]++;
    }
    ctab[c].time = time;
    ctab[c].lasttopnum = stab[t].num;
  }
}

int increment(const Set s, unsigned int k, int i1, int i2, int time,
	      int cladetops)
{
  /* s is the set of taxa. k is the hash value of the subtree.
     i1 and i2 are the indices into the subtree table of the left
     and right subtrees, respectively. */

  int c,t;
  unsigned int j;

  while(t=sindex[k])
    if(stab[t].left == i1 && stab[t].right == i2) {
      stab[t].count++;
      c = stab[t].clade;
      ctab[c].count++;
      addTrans(c,t,time,cladetops);
      return t;
    }
    else
      k = (k+1) % SSIZE;

  if(num_trees == MAXTREES) {
    cerr << "Error: Too many distinct subtrees." << endl;
    exit(0);
  }

  t = sindex[k] = ++num_trees;
  stab[t].left = i1;
  stab[t].right = i2;
  stab[t].count = 1;

  j = chash(s);
  while(c=cindex[j])
    if(setEqual(ctab[c].set,s)) {
      stab[t].clade = c;
      ctab[c].count++;
      stab[t].num = ++ctab[c].trees;
      stab[t].next = ctab[c].first;
      ctab[c].first = t;
      stab[t].name = 0;
      addTrans(c,t,time,cladetops);
      return t;
    }
    else
      j = (j+1) % CSIZE;

  if(num_clades == MAXCLADES) {
    cerr << "Error: Too many distinct clades." << endl;
    exit(0);
  }

  c = cindex[j] = ++num_clades;
  stab[t].clade = c;
  copySet(ctab[c].set,s);
  ctab[c].count = 1;
  ctab[c].trees = 1;
  stab[t].next = -1;
  ctab[c].first = t;
  stab[t].name = 0;
  ctab[c].name = 0;
  ctab[c].time = time;
  ctab[c].lasttopnum = stab[t].num = 1;
  if(time>1) 
    ctab[c].trans[0][1]++;
  return t;
}
  
char *storeTree (char *top, int time, Set s, unsigned int& k, int& i,
		  int cladetops)
{
  char *top2, *top3;
  Set s1,s2;
  unsigned int k1,k2;
  int i1,i2;

  if(*top == '\0')
    return top;
  if(*top == '(') {
    top2 = storeTree(top+1,time,s1,k1,i1,cladetops);
    if(*top2 == ',')
      top2++;
    else {
      cerr << "Error: Missing comma - /" << top << "/" << endl;
      exit(1);
    }
    top3 = storeTree(top2,time,s2,k2,i2,cladetops);
    if(*top3 != ')') {
      cerr << "Error: Mismatched parentheses - /" << top << "/" << endl;
      exit(1);
    }
    setUnion(s1,s2,s);
    k = SHASH(k1,k2);
    i = increment(s, k, i1, i2, time, cladetops);
    return top3+1;
  }
  k = 0;
  for(;isdigit(*top);top++)
    k = 10 * k + (*top-'0');
  if(k>MAXLEAVES) {
    cerr << "Error: taxa number too large - " << k << endl;
    exit(1);
  }
  i = -k;
  singleton(k,s);
  return top;
}

char *makeSet (char *top, Set s)
{
  char *top2, *top3;
  Set s1,s2;
  int k;

  if(*top == '\0')
    return top;
  if(*top == '(') {
    top2 = makeSet(top+1,s1);
    if(*top2 == ',')
      top2++;
    else {
      cerr << "Error: Missing comma - /" << top << "/" << endl;
      exit(1);
    }
    top3 = makeSet(top2,s2);
    if(*top3 != ')') {
      cerr << "Error: Mismatched parentheses - /" << top << "/" << endl;
      exit(1);
    }
    setUnion(s1,s2,s);
    return top3+1;
  }
  k = 0;
  for(;isdigit(*top);top++)
    k = 10 * k + (*top-'0');
  if(k>MAXLEAVES) {
    cerr << "Error: taxa number too large - " << k << endl;
    exit(1);
  }
  singleton(k,s);
  return top;
}

int cmpTops2Int(int a, int b)
{ int c;

  if(a==b) 
    return 0;
  if(a<0)
    if(b<0)
      return b-a;
    else
      return -1;
  else if(b<0)
    return 1;
  else {
    if(stab[a].name)
      if(stab[b].name)
	return ctab[stab[a].clade].name - ctab[stab[b].clade].name;
      else 
	return -1;
    else if(stab[b].name)
      return 1;
    else 
      return ((c = cmpTops2Int(stab[a].left,stab[b].left)) ? c :
	      cmpTops2Int(stab[a].right,stab[b].right));
  }
}

int cmpTops2(const void *a, const void *b)
{
  return (cmpTops2Int(*((int *)a),*((int *)b)));
}

int cmpTops(const void *a, const void *b)
{
  return stab[*((int *)b)].count - stab[*((int *)a)].count;
}
  

int cmpClades(const void *a, const void *b)
{
  return cmpSet(ctab[*((int *)b)].set,ctab[*((int *)a)].set);
}

void addClade(int i, int clades[], int& num)
{
  int j=0;

  while(j<num)
    if(subset(ctab[i].set,ctab[clades[j]].set))
      return;
    else if(subset(ctab[clades[j]].set,ctab[i].set))
      clades[j] = clades[--(num)];
    else
      j++;
  clades[num++] = i;
}

int printClades(int min, int maxsub, int clades[], Set universalSet, ofstream& out)
{
  unsigned int m;
  int i,j,k,l,c;
  int num=0;
  int tops[MAXCLADETOPS];
  Set allset;

  makeEmptySet(allset);
  for(m=0;m<CSIZE;m++)
    if((c=cindex[m]) && ctab[c].count >= min && ctab[c].trees <= maxsub) 
      addClade(c,clades,num);
  qsort(clades,num,sizeof(int),cmpClades);
  for(i=0;i<num;i++) {
    j = clades[i];
    ctab[j].name = i+1;
    setUnion(ctab[j].set,allset,allset);
    out << setw(6) << ctab[j].count << "  " << char(i+'A') << "    ";
    printSet(ctab[j].set,out);
    out << endl;
    for(k=ctab[j].first,l=0;k!=-1;k=stab[k].next,l++)
      tops[l] = k;
    qsort(tops,l,sizeof(tops[0]),cmpTops);
    for(k=0;k<l;k++) {
      out << setw(6) << stab[tops[k]].count << "  " << char(i+'A');
      int oldOptions = out.setf(ios::left);
      out << setw(3) << k+1 << " ";
      out.flags(oldOptions);
      printTree(tops[k],1,out);
      out << endl;
      stab[tops[k]].name = k+1;
    }
    out << endl;
  }
  setDifference(universalSet,allset,allset);
  printElements(allset,out);
  return num;
}

void printTrees(int trees[], int n, int total, int printId, ofstream& out)
{
  int sum = 0, k, m;

  qsort(trees,n,sizeof(int),cmpTops);
  for(k=0;k<n;k++) {
    m = stab[trees[k]].count;
    sum += m;
    out << setw(5) << m << " " << setw(6) << setprecision(3)
	 << m/(double)total << " " << setw(6) << sum/(double)total << " ";
    printTree(trees[k],printId,out);
    out << endl;
  }
}

void printTrans(const int clades[], int n, int total, ofstream& out)
{
  int i, j, k, s, sum;
  int zerorow, zerocol;
  char cname;
  int (*cladeMatrix)[MAXCLADETOPS+1];
  int num[MAXCLADETOPS+1];

  for(i=0;i<n;i++) {
    s = ctab[clades[i]].trees;
    cname = char(ctab[clades[i]].name+'A'-1);
    cladeMatrix = ctab[clades[i]].trans;
    for(k=ctab[clades[i]].first;k!=-1;k=stab[k].next)
      num[stab[k].name] = stab[k].num;
    out << "     | ";
    for(j=1;j<s+1;j++) {
      out << "  " << cname;
      int oldOptions = out.setf(ios::left);
      out << setw(3) << j;
      out.flags(oldOptions);
    }
    zerocol = zerorow = sum = 0;
    for(k=0;k<s+1;k++)
      for(j=0;j<s+1;j++)
	sum += cladeMatrix[k][j];
    cladeMatrix[0][0] = total - 1 - sum;
    for(k=0;k<s+1;k++) {
      zerocol = zerocol || cladeMatrix[k][0];
      zerorow = zerorow || cladeMatrix[0][k];
    }
    if(zerocol)
      out << "   -  ";
    out << "\n-----+";
    for(j=1;j<s;j++)
      out << "------";
    if(zerocol)
      out << "------";
    out << "-----";
    out << endl;
    for(k=1;k<s+1;k++) {
      out << " " << cname;
      int oldOptions = out.setf(ios::left);
      out << setw(3) << k << "|";
      out.flags(oldOptions);
      for(j=1;j<s+1;j++)
	out << setw(5) << cladeMatrix[num[k]][num[j]] << " ";
      if(zerocol)
	out << setw(5) << cladeMatrix[num[k]][0] << " ";
      out << endl;
    }
    if(zerorow) {
      out << "  -  |";
      for(j=1;j<s+1;j++)
	out << setw(5) << cladeMatrix[0][num[j]] << " ";
      if(zerocol)
	out << setw(5) << cladeMatrix[0][0] << " ";
      out << endl;
    }
    out << endl << endl;
  }
}

void addZeroTrans(int i, int time)
{
  int c;

  if(i>0) {
    c = stab[i].clade;
    if(ctab[c].time != time)
      ctab[c].trans[ctab[c].lasttopnum][0]++;
    addZeroTrans(stab[i].left,time);
    addZeroTrans(stab[i].right,time);
  }
}

void printCladeCount(int k, int total, ofstream& out)
{
  int c;

  if(k>=0) {
    c = stab[k].clade;
    out << setw(10) << ctab[c].count << " " << setw(6) << setprecision(3)
	 << (double)ctab[c].count/(double)total << " ";
    printSet(ctab[c].set,out);
    out << endl;
    printCladeCount(stab[k].left,total,out);
    printCladeCount(stab[k].right,total,out);
  }
}

void readFile(istream& f, char* firstLine, int& first, int n, int& total, 
	      Set set, unsigned int& k, int& j, int cladetops)
{
  char s[MAXLINE],c;
  for(int i=0;i<n;i++)
    if(f.getline(s,MAXLINE,'\n')==NULL) {
      cerr << "Error: Input contains " << (i==n-1 ? "exactly" : "fewer than ")
	   << n << " lines." << endl;
      exit(1);
    }
  while(f.get(s,MAXLINE,'\n')) {
    while((c=f.get())!=EOF && c !='\n')
      ;
    storeTree(s, ++total, set, k, j, cladetops);
    if(first) {
      strcpy(firstLine,s);
      first = 0;
    }
  }
}

void summarize(int n, double threshold, int cladetops,
	       char **names, int num_names, ofstream &out)
{
  char s[MAXLINE];
  int total=0;
  Set set,universalSet;
  int i,l,r;
  unsigned int k;
  unsigned int m;
  int trees[SSIZE];
  int clades[MAXLEAVES],num;
  char **name;
  int first = 1;

  for(name=names;name<names+num_names;name++) {
    if(strcmp(*name,"-")==0)
      readFile(cin,s,first,n,total,set,k,i,cladetops);
    else {
      ifstream f(*name);
      if(!f) {
      	mssg.Format("Error: Could not open %s\n",*name);
		PrintMssg();
		//cerr << "Error: Could not open " << *name << endl;
		exit(1);
      }
      readFile(f,s,first,n,total,set,k,i,cladetops);
      f.close();
    }
  }

  if(total==0) {
    mssg.Format("Error: Input contains exactly %d lines.\n",n);
	PrintMssg();
    //cerr << "Error: Input contains exactly " << n << " lines." << endl;
    exit(1);
  }

  addZeroTrans(i,total);
  makeSet(s,universalSet);
  out << "******************** Named clades "
       << "********************" << endl << endl;
  num = printClades(int(total*threshold),cladetops,clades,universalSet,out);
  out << endl;

  m = chash(universalSet);
  while(!(setEqual(ctab[cindex[m]].set,universalSet)))
      m = (m+1) % CSIZE;
  for(r=ctab[cindex[m]].first,l=0;r!=-1;r=stab[r].next,l++)
    trees[l] = r;
  out <<  "******************** Tree topologies "
       << "********************"  << endl << endl;
  out << "Count  Prob.  Cum.  Tree topology" << endl;
  printTrees(trees,l,total,1,out);
  out << endl;

  out << "***** Posterior probabilities of clades in most probable tree "
       << "topology *****" << endl << endl;
  out << "     Count  Prob. Tree topology" << endl;
  printCladeCount(trees[0],total,out);
  out << endl;

  out << "******************** Clade transition matrices "
       << "********************" << endl << endl;
  printTrans(clades,num,total,out);

  qsort(trees,l,sizeof(int),cmpTops2);
  for(r=1,i=0;r<l;r++)
    if(cmpTops2Int(trees[i],trees[r])==0)
      stab[trees[i]].count += stab[trees[r]].count;
    else
      trees[++i] = trees[r];

  out << "******************** Clade tree topologies "
       << "********************" << endl << endl;
  out << "Count  Prob.  Cum.  Tree topology" << endl;
  printTrees(trees,i+1,total,0,out);
}

/*
int main(int argc, char *argv[])
{
  int n=0,cladetops=MAXCLADETOPS,num_names=0;
  double threshold=DEFAULTTHRESHOLD;
  char *names[100],**p;
  int i;

  out.setf(ios::fixed, ios::floatfield);
  out.setf(ios::showpoint);
  cerr.setf(ios::fixed, ios::floatfield);
  cerr.setf(ios::showpoint);

  out << "******************** BAMBE Summarize Version " << VERSION 
       << " ********************" << endl << endl;
  out << "Invocation: ";
  for(i=0;i<argc;i++)
    out << argv[i] << " ";
  out << endl << endl;
  if(argc > 1) {
    for(p=argv+1;p<argv+argc;p++) {
      if(strcmp(*p,"-n")==0) {
	if(++p>=argv+argc)
	  usageError(argv[0]);
	istrstream f(*p);
	if(!(f >> n))
	  usageError(argv[0]);
      }
      else if(strcmp(*p,"-p")==0) {
	if(++p>=argv+argc)
	  usageError(argv[0]);
	istrstream f(*p);
	if(!(f >> threshold))
	  usageError(argv[0]);
	if(threshold < 0.5 || threshold > 1.0) {
	  cerr << "Error: threshold value (" << setprecision(3) 
	    << threshold << ") must be between 0.5 and 1.0." << endl;
	  exit(1);
	}
      }
      else if(strcmp(*p,"-c")==0) {
	if(++p>=argv+argc)
	  usageError(argv[0]);
	istrstream f(*p);
	if(!(f >> cladetops))
	  usageError(argv[0]);
	if(cladetops < 1 || cladetops > MAXCLADETOPS) {
	  cerr << "Error: Number of topologies (" << cladetops 
	       << ") of a named clade must be between 1 and " << MAXCLADETOPS
	       << "." << endl;
	  exit(1);
	}
      }
      else if(num_names<100)
	names[num_names++] = *p;
    }
    summarize(n,threshold,cladetops,names,num_names);
  }
  else 
    usageError(argv[0]);
  return 0;
}
*/

/* Old Summarize
void summarize(int n, double threshold, int cladetops,
	       char **names, int num_names, FILE *out)
{
  FILE *f;
  char s[MAXLINE];
  int total=0;
  Set set,universal_set;
  int i,j,l,r;
  unsigned int k;
  unsigned int m;
  int trees[SSIZE];
  int clades[MAXLEAVES],num;
  char **name;

	TRACE("num_clades = %d num_trees %d\n", num_clades, num_trees);
  
  for(name=names;name<names+num_names;name++) {
    if(strcmp(*name,"-")==0)
      f = stdin;
    else if(!(f=fopen(*name,"r"))) {
      	mssg.Format("Error: Could not open %s\n",*name);
		PrintMssg();
      //fprintf(stderr,"Error: Could not open %s\n",*name);
      exit(1);
    }
    for(i=0;i<n;i++)
      if(fgets(s,MAXLINE,f)==NULL) {
      	mssg.Format("Error: Input contains fewer than %d lines.\n",n);
		PrintMssg();
	//fprintf(stderr,"Error: Input contains fewer than %d lines.\n",n);
	exit(1);
      }
    while(fgets(s,MAXLINE,f))
      store_tree(s, ++total, set, &k, &i, cladetops);
    fclose(f);
  }

  if(total==0) {
      	mssg.Format("Error: Input contains exactly %d lines.\n",n);
		PrintMssg();
    //fprintf(stderr,"Error: Input contains exactly %d lines.\n",n);
    exit(1);
  }

  add_zero_trans(i,total);
  make_set(s,universal_set);
  fprintf(out,  "******************** Named clades "
	 "********************\n\n");
  num = print_clades((int)total*threshold,cladetops,clades,universal_set, out);
  fprintf(out,  "\n");

  m = chash(universal_set);
  while(!(set_equal(ctab[cindex[m]].set,universal_set)))
      m = (m+1) % CSIZE;
  for(r=ctab[cindex[m]].first,l=0;r!=-1;r=stab[r].next,l++)
    trees[l] = r;
  fprintf(out,  "******************** Tree topologies "
	 "********************\n\n");
  fprintf(out,  "Count  Prob.  Cum.  Tree topology\n");
  print_trees(trees,l,total,1, out);
  fprintf(out,  "\n");

  fprintf(out,  "***** Posterior probabilities of clades in most probable tree "
	 "topology *****\n\n");
  fprintf(out,  "     Count  Prob. Tree topology\n");
  print_clade_count(trees[0],total, out);
  fprintf(out,  "\n");

  fprintf(out,  "******************** Clade transition matrices "
	 "********************\n\n");
  print_trans(clades,num,total, out);

  qsort(trees,l,sizeof(int),cmp_tops2);
  for(r=1,i=0;r<l;r++)
    if(cmp_tops2_int(trees[i],trees[r])==0)
      stab[trees[i]].count += stab[trees[r]].count;
    else
      trees[++i] = trees[r];

  fprintf(out,  "******************** Clade tree topologies "
	 "********************\n\n");
  fprintf(out, "Count  Prob.  Cum.  Tree topology\n");
  print_trees(trees,i+1,total,0, out);
}
*/