// TOPView.cpp : implementation of the CTOPView class
//

#include "stdafx.h"
#include "BAMBE.h"
#include "Window.h"

#include "TOPView.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>		// for sqrt function
#include "CloseWins.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CCriticalSection g_cs;

extern Windows win;
extern CBAMBEApp* pBApp;
/////////////////////////////////////////////////////////////////////////////
// CTOPView

IMPLEMENT_DYNCREATE(CTOPView, CScrollView)

BEGIN_MESSAGE_MAP(CTOPView, CScrollView)
	//{{AFX_MSG_MAP(CTOPView)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_COMMAND(ID_RUN_STOP, OnRunStop)
	ON_UPDATE_COMMAND_UI(ID_RUN_STOP, OnUpdateRunStop)
	ON_WM_HSCROLL()
	ON_WM_SIZE()
	ON_COMMAND(ID_RUN_PAUSE, OnRunPause)
	ON_UPDATE_COMMAND_UI(ID_RUN_PAUSE, OnUpdateRunPause)
	ON_COMMAND(ID_RUN_RESUME, OnRunResume)
	ON_UPDATE_COMMAND_UI(ID_RUN_RESUME, OnUpdateRunResume)
	ON_COMMAND(ID_RUN_RUN, OnRunRun)
	ON_COMMAND(ID_CLOSEALLWINS, OnCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_CLOSEALLWINS, OnUpdateCloseAllWins)
	ON_UPDATE_COMMAND_UI(ID_RUN_RUN, OnUpdateRunRun)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_USERINVALTOP, OnUserInvalTop)
	ON_MESSAGE(WM_USERCLOSETOP, OnUserCloseTop)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTOPView construction/destruction

CTOPView::CTOPView() : m_ViewCharSize(0,0), m_DocSize(500, 500)
{
	m_pFont = NULL;
	m_TotalHeight = 0;
	m_largestWidth = 0;
	m_longestName = 0;
	if (!win.NameList->IsEmpty()) {
		POSITION pos;
		pos = win.NameList->FindIndex(0);
		int ct = win.NameList->GetCount();
		CString tmp;
		for (int i = 0; i < ct; i++) {
			tmp = win.NameList->GetAt(pos);
			if (tmp.GetLength() > m_longestName)
				m_longestName = tmp.GetLength();
		}
	}
	IsDrawing = FALSE;
	IsHScroll = FALSE;
}

CTOPView::~CTOPView()
{
	delete m_pFont;
}

BOOL CTOPView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTOPView drawing
void CTOPView::OnDraw(CDC* pDC)
{
  if (win.curTree != NULL) {
 
	IsDrawing = TRUE;
	pDC->SetMapMode(MM_LOENGLISH);
	CFont* pPreviousFont = pDC->SelectObject(GetFont());

	CPen pen(PS_SOLID,2,RGB(0,0,0));
	CPen* oldPen = pDC->SelectObject(&pen);

	CSize CharSize = GetCharSize();

	static PrintNodes pnodes(win.NumLeaves, win.NumNodes, win.Nodes);
	Internal* printT = (Internal*)(win.curTree->copyTopology(pnodes));

 	double scaleFact = 1000.0;
	double width;

	if (!IsHScroll) {
		CRect rc;   
		pDC->GetClipBox(&rc);
		width = rc.right - rc.left;

		double treeh, d;

		printT->diameter(treeh, d);		// ->root

		if (treeh > 0) {
			while ((scaleFact * treeh) < (width / 2)) {
				scaleFact *= sqrt(1.2);
			}	
			TRACE("scaleFact=%f   ", scaleFact);

			while ((scaleFact * treeh) > (width * .75)) {
				scaleFact /= sqrt(1.2);
			}
			TRACE("scaleFact=%f   ", scaleFact);
		}

		TRACE("!IsHScroll, treeh = %f, width = %f\n", treeh, width);
		m_scalef = scaleFact;
	}
	else scaleFact = m_scalef;

	int ry, h, dgap = CharSize.cy * 2 + .5;
	m_TotalHeight = win.NumLeaves * (CharSize.cy * 2 + .5); // so can leave two spaces between lines

	printT->setMinTaxa();	// root->

	int header = -10;
	if (pDC->IsPrinting()) {
		PrintHeader(pDC);
		header = -40;
	}

	TRACE("\nx=0.007, y=%d, scaleFactor=%f, ry=%d, h=%d, dgap=%d\n", header, scaleFact, ry, h, dgap);
	DrawTreeInt(pDC, printT, 0.0001, header, scaleFact, ry, h, dgap);
	
	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}
	if (oldPen) {
		pDC->SelectObject(oldPen);
	}

	//printT->deleteTree();

	IsDrawing = FALSE;
  }
}

void CTOPView::PrintHeader(CDC * pDC)
{
	CString tmp;
	CGenericDoc* pDoc = GetDocument();
/*	int pageWd = pDC->GetDeviceCaps(HORZRES);
	int logPixels = pDC->GetDeviceCaps(LOGPIXELSX);// finds dots per inch
	double pi = logPixels / 100;

	double posx, mp, bot;
	bot = (1.0/pi);
	mp = (4.0/3.0)/bot;*/

	CFont* pPreviousFont = pDC->SelectObject(GetFont());
	pDC->SetMapMode(MM_LOENGLISH);

//	for (int i = m_nPage; i <= m_nMax; i++) { 
		tmp.Format("%s", (LPCSTR) pDoc->GetTitle());
//		posx = 20 + ((i-1) * (pageWd/mp) * (4.0/3.0));
		double posx = 20;
 		pDC->TextOut(posx, -10, tmp);
//	}

	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
	}
}

void CTOPView::OnUpdate()
{
	ComputeViewMetrics();
  
	SetScrollSizes(MM_LOENGLISH, GetDocSize());

	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CTOPView printing

BOOL CTOPView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTOPView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo)
{
	int pageHt = pDC->GetDeviceCaps(VERTRES);
	int logPixelsY = pDC->GetDeviceCaps(LOGPIXELSY);// finds dots per inch

	int numInches = (int)((m_DocSize.cy + 40) / 100);

	int numPages = (numInches * logPixelsY) / pageHt + 1;

	pInfo->SetMaxPage(numPages);
}

void CTOPView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CTOPView diagnostics

#ifdef _DEBUG
void CTOPView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CTOPView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CGenericDoc* CTOPView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGenericDoc)));
	return (CGenericDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTOPView message handlers

CFont*	CTOPView::GetFont()
{
	if(m_pFont == NULL) {
		m_pFont = new CFont;
		if(m_pFont) {
			//	Default to 10 pt Arial
			m_pFont->CreatePointFont(100, "Times New Roman");
		}
	}
	return m_pFont;
}

void CTOPView::ComputeViewMetrics()
{	
		// get a CDC* for the screen
	CDC* pDC = CDC::FromHandle(::GetDC(NULL));
	int nSaveDC = pDC->SaveDC();
  
		// select mapping mode
	pDC->SetMapMode(MM_LOENGLISH); 
  
		// select the font and get its metrics
	CFont* pPreviousFont = pDC->SelectObject(GetFont());
	TEXTMETRIC tm;
	pDC->GetTextMetrics(&tm);
  
		//	Calculate view character size
	m_ViewCharSize.cy = tm.tmHeight + tm.tmExternalLeading;
	m_ViewCharSize.cx = tm.tmAveCharWidth; 
  
		// convert to device units to minimize round off error
	pDC->LPtoDP(&m_ViewCharSize);
  
		//	Calculate document size
	CGenericDoc* pDoc = GetDocument();

	if (m_DocSize.cy < m_TotalHeight) {
		m_DocSize.cy = m_TotalHeight;
	}
	if (m_DocSize.cx < m_largestWidth) {
		m_DocSize.cx = m_largestWidth;
	}
  
		// clean up
	if(pPreviousFont) {
		pDC->SelectObject(pPreviousFont);
    }
	pDC->RestoreDC(nSaveDC);
	::ReleaseDC(NULL,pDC->GetSafeHdc());
}

void CTOPView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	SetScrollSizes(MM_LOENGLISH, GetDocSize());
	ComputeViewMetrics();
	pBApp->globals.g_TopHwnd = GetSafeHwnd();
}

LRESULT CTOPView::OnUserInvalTop(WPARAM wParam, LPARAM lParam)
{
	if (!IsDrawing) 
		OnUpdate();

	return 1;
}

LRESULT CTOPView::OnUserCloseTop(WPARAM wParam, LPARAM lParam)
{
	CGenericDoc* pDoc = GetDocument();
	pDoc->OnCloseDocument();
	return 1;
}

void CTOPView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CTOPView::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CTOPView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CTOPView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

void CTOPView::DrawTreeInt(CDC *pDC, Node* root, double x, int y, double sf, 
						   int& ry, int& h, int dgap)
{
	if (root->isLeaf()) {
		DrawName(pDC, x*sf, y, root);
		ry = y;
		h = y;
	}
	else {
		int lry, lh, rry;
		if (root->getLeft()->getMinTaxa() < root->getRight()->getMinTaxa()) {
			DrawTreeInt(pDC, (root->getLeft()), x + root->getLeft()->getBranchLength(), y, sf, lry, lh, dgap);
			DrawTreeInt(pDC, (root->getRight()), x + root->getRight()->getBranchLength(), lh - dgap, sf, rry, h, dgap);
		}
		else{
			DrawTreeInt(pDC, (root->getRight()), x + root->getRight()->getBranchLength(), y, sf, rry, lh, dgap);
			DrawTreeInt(pDC, (root->getLeft()), x + root->getLeft()->getBranchLength(), lh - dgap, sf, lry, h, dgap);
		}
		
		int xp = x * sf + .5;
		DrawLine(pDC, xp, lry, (x + root->getLeft()->getBranchLength()) * sf, lry);
		DrawLine(pDC, xp, rry, (x + root->getRight()->getBranchLength()) * sf, rry);
		DrawLine(pDC, xp, lry, xp, rry);
		ry = (lry + rry) / 2;
	}
}

void CTOPView::DrawName(CDC* pDC, int x, int y, Node *root)
{
	if (win.NameList) {
		if (!win.NameList->IsEmpty()) {	
			POSITION pos;
			int index =	root->getNumber();
			pos = win.NameList->FindIndex(index);
			pDC->TextOut(x + 5, y + 8, win.NameList->GetAt(pos));
		}
	}
}

void CTOPView::DrawLine(CDC* pDC, int xp, int yp, int newx, int newy)
{
	//TRACE("xp=%d, yp=%d, newx=%d, newy=%d\n", xp, yp, newx, newy);
	pDC->MoveTo(xp, yp);
	pDC->LineTo(newx, newy);
	if (newx > m_largestWidth) {
		m_largestWidth = newx + (m_longestName * m_ViewCharSize.cy);
	}
}

void CTOPView::OnRunStop() 
{
	pBApp->globals.IsStop = TRUE;
	pBApp->globals.RunAvail = TRUE;
	pBApp->globals.IsPause = FALSE;
}

void CTOPView::OnUpdateRunStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.RunAvail);			
}

void CTOPView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	IsHScroll = TRUE;
	CScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
	IsHScroll = FALSE;
}

void CTOPView::OnSize(UINT nType, int cx, int cy) 
{
	CScrollView::OnSize(nType, cx, cy);

	OnUpdate();
}

void CTOPView::OnRunPause() 
{
	pBApp->globals.IsPause = TRUE;
}

void CTOPView::OnUpdateRunPause(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!pBApp->globals.IsPause && !pBApp->globals.RunAvail);		
}

void CTOPView::OnRunResume() 
{
	pBApp->globals.IsPause = FALSE;	
}

void CTOPView::OnUpdateRunResume(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.IsPause && !pBApp->globals.RunAvail);		
}

void CTOPView::OnRunRun() 
{
	pBApp->globals.g_pView->SendMessage(WM_USERCHILDRUN);	
}

void CTOPView::OnUpdateRunRun(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail && pBApp->globals.DocIsOpen);			
}

void CTOPView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if (pDC->IsPrinting()) {
		int pageHt = pDC->GetDeviceCaps(VERTRES);
		int originY = pageHt * (pInfo->m_nCurPage - 1);
		pDC->SetViewportOrg(0, -originY);
		CView::OnPrepareDC(pDC, pInfo);
	}
	else {
		CScrollView::OnPrepareDC(pDC, pInfo);
	}
}

void CTOPView::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);			
}

void CTOPView::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(pBApp->globals.RunAvail || pBApp->globals.IsPause);			
}

void CTOPView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	m_nPage = pInfo->m_nCurPage;
	m_nMax = pInfo->GetMaxPage();
		
	CScrollView::OnPrint(pDC, pInfo);
}

void CTOPView::OnCloseAllWins()
{
	CCloseWins printmssg;
	int ans = printmssg.DoModal();

	if(ans == IDOK)
		pBApp->globals.CloseAllWins();
}

void CTOPView::OnUpdateCloseAllWins(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}
